###############################################################################
# (c) Copyright 2022 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
import glob
import csv
import math
import datetime
from ruamel.yaml import YAML
import sys
import re

arguments = []
if not sys.stdin.isatty():
    for line in sys.stdin:
        for word in line.split():
            #print(word)
            arguments.append(word)

count = len(arguments)
#print(count)
#print(arguments)

# You can pipe in to stdin to change tiltName and n_it
# echo -n <tiltName> <n_it> | Panoptes/run gaudirun.py ...
# OR
# must have pipe in to gaudirun.py to change n_it only
# echo -n <n_it> | Panoptes/run gaudirun.py ...
# otherwise tiltName = "" and n_it = 0

# Since we are doing this by hand, specify the iteration number in stdin
n_it = 0
if count == 2 and arguments[1] in [
        '0', '1', '2', '3', '4', '5', '6', '7', '8'
]:
    n_it = int(arguments[1])
if count == 1 and arguments[0] in [
        '0', '1', '2', '3', '4', '5', '6', '7', '8'
]:
    n_it = int(arguments[0])
print("Iteration: " + str(n_it))

# possible tiltname can be given via stdin
tiltName = ""  # uses 'alignment2022'
if count == 2 and arguments[0] in [
        "priYn", "priYp", "priZn", "priZp", "secYn", "secYp", "secZn", "secZp"
]:
    tiltName = arguments[0]
#tiltName = "priYn" # uses '2022_12_06_Rich1_priYn', a branch of 'alignment2022'
#tiltName = "priYp" # uses '2022_12_06_Rich1
#tiltName = "priZn" # uses '2022_12_06_Rich1
#tiltName = "priZp" # uses '2022_12_06_Rich1
#tiltName = "secYn" # uses '2022_12_06_Rich1
#tiltName = "secYp" # uses '2022_12_06_Rich1
#tiltName = "secZn" # uses '2022_12_06_Rich1
#tiltName = "secZp" # uses '2022_12_06_Rich1
print("Tilt Name: " + tiltName)

from GaudiKernel.SystemOfUnits import (GeV, MeV, mm, mrad)
from Moore import (
    options,
    run_reconstruction,
)

from RecoConf.rich_data_monitoring import (
    default_rich_monitoring_options,
    alignment_rich_monitoring_options,
)

from PyConf.Algorithms import (PrForwardTrackingVelo,
                               VPRetinaFullClusterDecoder)
from PyConf.Algorithms import VeloRetinaClusterTrackingSIMD
from RecoConf.hlt1_tracking import make_VeloClusterTrackingSIMD
from RecoConf.hlt1_tracking import (make_reco_pvs, make_PatPV3DFuture_pvs,
                                    make_velo_full_clusters)

from RecoConf.rich_reconstruction import default_rich_reco_options

from Panoptes.alignment import (
    standalone_rich_online_align_reco,
    standalone_rich_panel_align_reco,
)

# from Configurables import (
#      UpdateManagerSvc
# )
"""Options for running over data with FT raw bank version 6."""
from RecoConf.decoders import (
    default_ft_decoding_version, )

useRealData = True

options.n_threads = 1
options.evt_max = 25000
if useRealData is not True:
    options.set_input_and_conds_from_testfiledb(
        'upgrade_Sept2022_minbias_0fb_md_xdigi')
    options.input_type = 'ROOT'
else:
    # now we put this on the command line
    # exec(open("./data_255358.py").read())

    # load one of the five data samples that Roel used
    # options.input_files = ["/eos/lhcb/hlt2/LHCb/0000252186/Run_0000252186_HLT20319_20221104-114329-342.mdf"]

    options.simulation = False
    options.input_type = "MDF"

useDD4Hep = True

if useDD4Hep:
    from Configurables import LHCb__Det__LbDD4hep__DD4hepSvc as DD4hepSvc
    from DDDB.CheckDD4Hep import UseDD4Hep
else:
    UseDD4Hep = False

print("Using DD4Hep: ", UseDD4Hep)

whichRich = 1
rich = 'rich' + str(whichRich)
radiator = "Rich" + str(whichRich) + "Gas"

if UseDD4Hep:
    from Configurables import DDDBConf

    dd4hep = DD4hepSvc(
        DetectorList=["/world", "Magnet", "UT", "VP", "FT", "Rich1", "Rich2"])
    dd4hep.ConditionsLocation = 'git:/cvmfs/lhcb.cern.ch/lib/lhcb/git-conddb/lhcb-conditions-database.git'
    dd4hep.OutputLevel = 1

    if useRealData:
        options.conddb_tag = 'master'  # used for real data
        options.dddb_tag = 'run3/trunk'
        DDDBConf().GeometryVersion = 'run3/trunk'  # used for real data

        # latest scifi and velo alignments
        dd4hep.UseConditionsOverlay = True
        if (tiltName == "" and n_it == 0):
            dd4hep.ConditionsVersion = 'alignment2022'
        elif (n_it >= 1):
            dd4hep.ConditionsVersion = '2022_12_06_Rich' + str(
                whichRich) + '_i' + str(n_it) + "_new"
        else:
            dd4hep.ConditionsVersion = '2022_12_06_Rich' + str(
                whichRich) + '_' + tiltName

    else:
        options.conddb_tag = 'jonrob/all-pmts-active'  # used for DetDesc MC samples with DD4Hep
        DDDBConf(
        ).GeometryVersion = 'run3/before-rich1-geom-update-26052022'  # used for DetDesc MC samples with DD4Hep

else:
    options.conddb_tag = 'upgrade/sim-20220612-vc-md100-RICHcustomFTv6'
    options.simulation = True

param_list = ['p0_x', 'p1_x', 'p0_y', 'p1_y', 'p0_z', 'p1_z']
param_dict = dict.fromkeys(param_list)

# Only edit bad_align to play with alignment parameters
perf_align = [0.0, 0.0, -30.15, 30.15, 0.0, 0.0]
bad_align = [0.0, 0.0, -30.15, 30.15, 0.0, 0.0]
for row, param in zip(perf_align, param_dict.keys()):
    param_dict[param] = float(row)

if UseDD4Hep:
    pass

else:
    UpdateManagerSvc().ConditionsOverride += [
        f"Conditions/Alignment/Rich1/PDPanel0_Align :=  double_v dPosXYZ = {param_dict['p0_x']} {param_dict['p0_y']} {param_dict['p0_z']};",
        f"Conditions/Alignment/Rich1/PDPanel1_Align :=  double_v dPosXYZ = {param_dict['p1_x']} {param_dict['p1_y']} {param_dict['p1_z']};",
    ]

default_moni_opts = {}
default_reco_opts = {}
align_opts = {}

# Reco opts
wider_bkg = {"PhotonSelection": 'None'}
default_reco_opts.update(wider_bkg)
panel_select = {"ActivatePanel": (True, True)}
default_reco_opts.update(panel_select)

# Monitoring opts
tighter_minp = {
    "TightTrackSelection": {
        "MinP": 30. * GeV,
        "MinPt": 0.5 * GeV,
        "MaxChi2": 2.0,
        "MaxGhostProb": 0.1,
    }
}
default_moni_opts.update(tighter_minp)
wider_histo = {'CKResHistoRange': (0.025, 0.005, 0.004)}
default_moni_opts.update(wider_histo)
useUT = {'UseUT': False}
default_moni_opts.update(useUT)

PrForwardTrackingVelo.global_bind(
    MinQuality=0.0,
    DeltaQuality=0.0,
    DecisionLDA=-99.,
    MinTotalHits=9,
    MaxChi2PerDoF=50.,
    MaxChi2XProjection=60.,
    MaxChi2PerDoFFinal=28.,
    MaxChi2Stereo=16.,
    MaxChi2StereoAdd=16.,
)

make_VeloClusterTrackingSIMD.global_bind(
    algorithm=VeloRetinaClusterTrackingSIMD)
make_velo_full_clusters.global_bind(
    make_full_cluster=VPRetinaFullClusterDecoder)
make_reco_pvs.global_bind(make_pvs_from_velo_tracks=make_PatPV3DFuture_pvs)

mirror_align_tasks = [
    'Produce',  # fill the production set of histograms
    #'Monitor',   # add various checking histograms
    #'Map',       # add counters for creation of the HLT1 pre-selection line "map"
    #'Optimize',  # add counters for optimization of the RICH2 mirror combinations subset
    #'Select',    # check filling the rest of RICH2 miralignment_rich_reco_optionsror combinations along with 8 poorest
    #'Calibrate', # check filling all RICH2 mirror combinations with elimination when filled
    # 'Explore',   # explore influence of RICH1 coordinate systems on distribution shapes
]

if 'md' in options.conddb_tag:
    mp = 'md'
elif 'mu' in options.conddb_tag:
    mp = 'mu'
else:
    mp = 'Collision22'

n_kevts = str(math.trunc(options.evt_max / 1000)) + 'k'

#addinfo = '_minp60_'+mp+n_kevts
#addinfo = '_'+mp+n_kevts
addinfo = '_minp30_subset_' + mp + "_" + n_kevts
#addinfo = '_minp40_subset_' + mp + n_kevts
#addinfo = '_minp60_subset_theta2_'+mp+n_kevts
#addinfo = ''

# cumulative substring with all tasks chosen
tasks = ''
for task in mirror_align_tasks:
    tasks += '_' + task

# create timestamp
# now = datetime.datetime.now()
# dt = now.isoformat(timespec='minutes')  # Anatoly's old timestamp
# dt = dt.replace(":", "-")  # Anatoly's new timestamp
# retrieve the timestamp
yaml = YAML()
with open("./" + 'rich' + str(whichRich) + '_session_timestamp.yml') as inp:
    ts = yaml.load(inp)

# form the current variant name
current_variant = ts + '_' + 'rich' + str(whichRich) + addinfo + tasks

if 'subset' in current_variant:
    central_mirr_combs = {
        "PrebookHistos": [
            ('p00', 's03'),
            ('p01', 's06'),
            ('p02', 's12'),
            ('p03', 's09'),
        ],
    }
    align_opts.update(central_mirr_combs)

if 'minp40' in current_variant:
    align_opts.update({"MinP4Align": 40. * GeV})
elif 'minp30' in current_variant:
    align_opts.update({"MinP4Align": 30. * GeV})
elif 'minp60' in current_variant:
    align_opts.update({"MinP4Align": 60. * GeV})

if 'theta2' in current_variant:
    align_opts.update({"DeltaThetaRange": 0.002})

align_opts.update({"PoorestPopulation": 279.})

if (tiltName == ""):
    connectStr = ""
else:
    connectStr = "_"

# save output file with histograms
if UseDD4Hep:
    #options.histo_file = 'rich1_opt_dd4hep.root'
    options.histo_file = "./" + current_variant + "_histos" + connectStr + tiltName + "_i" + str(
        n_it) + ".root"
else:
    options.histo_file = 'rich1_opt.root'

# prepare part of the overridden options in use
# to be recorded into a YAML file
align_opts_dump = alignment_rich_monitoring_options(
    radiator=radiator, init_override_opts=align_opts)

# add overridden default_rich_monitoring_options
# to the part of the options in use to be recorded into the YAML file
align_opts_dump.update(
    default_rich_monitoring_options(init_override_opts=default_moni_opts))

# retrieve the timestamp
ts = re.findall(r'[\d]{4}-[\d]{2}-[\d]{2}T[\d]{2}-[\d]{2}', current_variant)[0]

# prepare options in use to be recorded into the YAML file
opts_dump = {}
opts_dump[ts] = {}
opts_dump[ts]['reco_opts'] = align_opts_dump

# append YAML file with the part of the options in use
with open("./" + rich + '_reco_opts.yml', 'a') as out:
    yaml.dump(opts_dump, out)

with standalone_rich_online_align_reco.bind(RichGas=radiator, MirrorAlignTasks = mirror_align_tasks,\
                                            noUT=True), \
    alignment_rich_monitoring_options.bind(radiator=radiator, \
                                            init_override_opts=align_opts), \
    default_ft_decoding_version.bind(value=6), \
    default_rich_reco_options.bind(init_override_opts=default_reco_opts), \
    default_rich_monitoring_options.bind(init_override_opts=default_moni_opts):
    run_reconstruction(options, standalone_rich_online_align_reco)
