###############################################################################
# (c) Copyright 2022 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
import os
import re
import sys
import subprocess
import csv
import argparse
import random

import sys
import numpy as np
from ROOT import gROOT, TH1D, TCanvas, TFile, TF1, gStyle, TGraph, TMultiGraph, TLine, TH2D, TExec
from scipy.optimize import minimize
import json
# Set batch mode so histograms aren't printed
#sys.argv.append('-b')
gROOT.SetBatch(True)


class AlignMonitor:
    def __init__(self, whichRich, floated
                 #            _alignConf,
                 #            _maxIt,
                 ):
        '''
        self.alignConf = _alignConf
        self.nameStr = self.alignConf.getProp('nameStr')
        self.whichRich = self.alignConf.getProp('Rich')
        self.displayMode = self.alignConf.getProp('displayMode')
        self.warningFactor = self.alignConf.getProp('warningFactor')
        self.maxIt = _maxIt  # the maximum labeled number of the iterations that we are looping through
        '''
        self.workdir = './'
        self.whichRich = whichRich
        self.maxIt = 0
        self.floated_str = '_'.join(str(round(f, 2)) for f in floated)
        self.prename = f'{prepath}/rich{self.whichRich}_opt_dd4hep_p{whichPanel}.root'

        # RICH1 settings
        self.maxpri = 3
        self.binToPriMirr = [[2, 3], [0, 1]]
        self.maxsec = 15
        self.binToSecMirr = [[10, 11, 14, 15], [8, 9, 12, 13], [2, 3, 6, 7],
                             [0, 1, 4, 5]]
        self.fitMin = -0.007
        self.fitMax = 0.007
        # if RICH2, use RICH2 settings
        if (self.whichRich == 2):
            self.maxpri = 55
            self.binToPriMirr = [[3, 2, 1, 0, 31, 30, 29, 28],
                                 [7, 6, 5, 4, 35, 34, 33, 32],
                                 [11, 10, 9, 8, 39, 38, 37, 36],
                                 [15, 14, 13, 12, 43, 42, 41, 40],
                                 [19, 18, 17, 16, 47, 46, 45, 44],
                                 [23, 22, 21, 20, 51, 50, 49, 48],
                                 [27, 26, 25, 24, 55, 54, 53, 52]]
            self.maxsec = 39
            self.binToSecMirr = [[3, 2, 1, 0, 23, 22, 21, 20],
                                 [7, 6, 5, 4, 27, 26, 25, 24],
                                 [11, 10, 9, 8, 31, 30, 29, 28],
                                 [15, 14, 13, 12, 35, 34, 33, 32],
                                 [19, 18, 17, 16, 39, 38, 37, 36]]
            # Original fit range
            # self.fitMin = -0.0039
            # self.fitMax = 0.0035
            self.fitMin = -0.0025
            self.fitMax = 0.0025
        self.isInsane = False
        self.finalCKres = None
        self.finalCKresErr = None

    def performMonitoring(self, p):
        import os
        gStyle.SetOptFit(1111)
        gStyle.SetOptStat(000000000)

        # File for the RICH piquet AND for the RICH mirror alignment experts
        if num_chunks != 0:
            expertFile = f'Rich{self.whichRich}_{survey_type}_{p}_({self.floated_str}).pdf'
        else:
            if os.path.isdir(self.workdir + "plots") is not True:
                os.mkdir(self.workdir + "plots")

            expertFile = self.workdir + f'plots/Rich{self.whichRich}_{survey_type}_{p}_({self.floated_str}).pdf'
        # File for the Alignment piquet ("duplicating" what goes to the Presenter for the Data Manager [except CK angle res stuff])
        monitorFile = self.workdir + f'Rich{self.whichRich}_AlignMonitor.pdf'
        # File with for testing purposes
        testingFile = self.workdir + f'Rich{self.whichRich}_AlignTesting.pdf'
        # Data Manager can only get histograms sent via self.monSvc

        theCanvas = TCanvas('theCanvas', 'MirrAlign')
        theCanvas.Divide(2, 2, 0.001, 0.001)

        theCanvas2 = TCanvas('theCanvas2', 'MirrAlignTest')
        theCanvas2.Divide(2, 2, 0.001, 0.001)

        theCanvas2.Clear()
        theCanvas.Clear()

        # generate resHistograms

        if self.maxIt == 0:
            pass
        elif self.maxIt < 2:
            theCanvas.Divide(2, 1, 0.0025, 0.0025)
        elif self.maxIt < 3:
            theCanvas.Divide(3, 1, 0.0025, 0.0025)
        elif self.maxIt < 4:
            theCanvas.Divide(2, 2, 0.0025, 0.0025)
        elif self.maxIt < 6:
            theCanvas.Divide(3, 2, 0.0025, 0.0025)
        elif self.maxIt < 8:
            theCanvas.Divide(4, 2, 0.0025, 0.0025)
        elif self.maxIt < 9:
            theCanvas.Divide(3, 3, 0.0025, 0.0025)
        else:
            theCanvas.Divide(4, 4, 0.0025, 0.0025)

        # reserve spaces for resHistograms
        resHistograms = [None] * (self.maxIt + 1)
        polbkg = [None] * (self.maxIt + 1)
        histFunc = [None] * (self.maxIt + 1)
        hist = [None] * (self.maxIt + 1)
        filename = [None] * (self.maxIt + 1)
        thisFile = [None] * (self.maxIt + 1)
        title = [None] * (self.maxIt + 1)
        fitRes = [None] * (self.maxIt + 1)

        # define resHistoTrend
        resHistoTrend = TH1D(
            'resHistoTrend', 'RICH' + str(self.whichRich) +
            ' Cherenkov angle resolution (mrad) per It. ', self.maxIt + 1,
            -0.5, self.maxIt + 0.5)

        for j in reversed(range(0, self.maxIt + 1)):
            filename[j] = self.prename
            if os.path.exists(filename[j]):
                thisFile[j] = TFile(filename[j], 'read')
                hist[j] = (thisFile[j].Get(
                    f'RICH/RiCKResLongTight/Rich{self.whichRich}Gas/ckResAll')
                           ).Clone()

                if (j == 0) and (j == self.maxIt):
                    hist[j].SetName('resHisto0')
                    hist[j].SetName('resHistoN')
                elif (j == self.maxIt):
                    hist[j].SetName('resHistoN')
                elif (j == 0):
                    hist[j].SetName('resHisto0')
                if (j != 0):
                    hist[j].SetName('resHisto' + str(j))

                fitRes[j] = self.fitCherenkovAngle(hist[j])
                resHistograms[j] = fitRes[j][0]
                resHistoTrend.SetBinContent(j + 1, fitRes[j][1] * 1000)
                resHistoTrend.SetBinError(j + 1, fitRes[j][2] * 1000)
                print("_i" + str(j) + " CK angle resolution:     " +
                      str(fitRes[j][1] * 1000) + " mrad.")
                print("_i" + str(j) + " CK angle resolution err: " +
                      str(fitRes[j][2] * 1000) + " mrad.")
                if j == self.maxIt:
                    self.finalCKres = fitRes[j][1] * 1000
                    self.finalCKresErr = fitRes[j][2] * 1000

                title[j] = 'RICH' + str(
                    self.whichRich) + ' Cherenkov angle resolution It. ' + str(
                        j)
                resHistograms[j].SetTitle(title[j])
                resHistograms[j].SetXTitle('#Delta#theta_{Cherenkov} (rad)')
                resHistograms[j].SetYTitle('Entries')

                histFunc[j] = resHistograms[j].GetFunction(
                    "Rich" + str(self.whichRich) + "fFitF3")
                polbkg[j] = TF1("polBKG" + str(j), "pol3(0)", self.fitMin,
                                self.fitMax)
                try:
                    polbkg[j].SetParameter(0, histFunc[j].GetParameter(3))
                    polbkg[j].SetParameter(1, histFunc[j].GetParameter(4))
                    polbkg[j].SetParameter(2, histFunc[j].GetParameter(5))
                    polbkg[j].SetParameter(3, histFunc[j].GetParameter(6))
                except:
                    polbkg[j].SetParameter(0, 5.0)
                    polbkg[j].SetParameter(1, 5.0)
                    polbkg[j].SetParameter(2, 5.0)
                    polbkg[j].SetParameter(3, 5.0)

                polbkg[j].SetLineColor(4)

                gStyle.SetStatW(0.11)

                theCanvas2.cd()
                resHistograms[j].Draw()
                polbkg[j].Draw("SAME")

                theCanvas2.Print(expertFile)

                theCanvas.cd()
                theCanvas.cd(1 + j)
                resHistograms[j].Draw()
                polbkg[j].Draw("SAME")

                thisFile[j].Close("R")
            else:
                print("WARNING: " + filename[j] + ": does not exist")

        #theCanvas.Print(monitorFile)

        # sys.stdout.flush()

        # Make resHistoTrend iteration trend plot
        resHistoTrend.SetMarkerStyle(8)
        resHistoTrend.SetXTitle('iteration number')
        resHistoTrend.SetYTitle(
            '#splitline{Cherenkov angle resolution (mrad)}{}')

        theCanvas2.Clear()
        theCanvas.Clear()
        resHistoTrend.DrawCopy()
        # sys.stdout.flush()

    def fitCherenkovAngle(self, hist):
        richStr = 'Rich' + str(self.whichRich)
        m_maxErrorForOK = 0.01

        xPeak = hist.GetBinCenter(hist.GetMaximumBin())
        delta = 0
        if (richStr == 'Rich1'):
            delta = 0.0025
        else:
            delta = 0.00105

        fitMin = xPeak - delta
        fitMax = xPeak + delta

        preFitFName = richStr + "PreFitF"
        preFitF = TF1(preFitFName, "gaus", fitMin, fitMax)
        preFitF.SetParameter(1, -0.0002)

        if (richStr == 'Rich1'):
            preFitF.SetParameter(2, 0.0008)
        else:
            preFitF.SetParameter(2, 0.0007)

        hist.Fit(preFitF, "RQ")
        fitOK = False

        lastFitF = preFitF
        bestFitF = None
        nPolFull = 3
        bestNPol = 0

        fFitF = None

        for nPol in range(1, nPolFull + 1):
            fFitFName = richStr + "fFitF" + str(nPol)
            fFitFType = "gaus(0)+pol" + str(nPol) + "(3)"
            fFitF = TF1(fFitFName, fFitFType, self.fitMin, self.fitMax)

            fFitF.SetParName(0, "Gaus Constant")
            fFitF.SetParName(1, "Gaus Mean")
            fFitF.SetParName(2, "Gaus Sigma")
            nParamsToSet = 3
            if (nPol > 1):
                nParamsToSet = 3 + nPol

            for p in range(0, nParamsToSet):
                fFitF.SetParameter(p, lastFitF.GetParameter(p))

            hist.Fit(fFitF, "RFQM")
            lastFitF = fFitF

            fitOK = fFitF.GetParError(1) < m_maxErrorForOK
            if fitOK:
                bestFitF = fFitF
                bestNPol = nPol
                print("fit ok", fFitF, nPol)
            else:
                if (nPol == nPolFull):
                    hist.Fit(fFitF, "RFSE0QM")
                    fitOK = True
                if nPol > 1:
                    break

        fitRes = [hist, fFitF.GetParameter(2), fFitF.GetParError(2)]
        return fitRes


def write_to_csv(x):
    with open(f'rich{whichRich}_{survey_type}_params_p{whichPanel}.csv',
              'w') as f:
        wtr = csv.writer(f)
        for axis in x.keys():
            wtr.writerow([x[axis]])


def check_if_already_run(x, csv_name):

    conditions = []
    for axis in x.keys():
        if x[axis] is not None:
            conditions.append(f'{axis}{x[axis]}')

    conds_to_write = ':'.join(str(d) for d in conditions)

    if (os.path.exists(
            f'ck_vs_param_R{whichRich}_{survey_type}_{csv_name}.csv')):
        reconstruction_already_run = False
        with open(f'ck_vs_param_R{whichRich}_{survey_type}_{csv_name}.csv',
                  'r') as f:
            reader = csv.reader(f, delimiter=',')
            for row in reader:

                if conds_to_write == row[1]:
                    reconstruction_already_run = True

                if reconstruction_already_run:
                    print("Reconstruction has already completed. Skipping...")
                    return reconstruction_already_run

        return reconstruction_already_run
    else:
        print(
            f'ck_vs_param_R{whichRich}_{survey_type}_{csv_name}.csv , does not exist. This must be the 1st time you have run this configuration.'
        )
        return


def check_for_reco_issues(finalCKres, finalCKresErr, csv_name):

    issue_present = False
    if (os.path.exists(
            f'ck_vs_param_R{whichRich}_{survey_type}_{csv_name}.csv')):

        with open(f'ck_vs_param_R{whichRich}_{survey_type}_{csv_name}.csv',
                  'r') as f:
            reader = csv.reader(f, delimiter=',')
            for row in reader:
                # check for bug where results are repeated
                if float(finalCKres) == float(
                        row[0]) and float(finalCKresErr) == float(row[2]):
                    issue_present = True
                    print(
                        "Repeated result in this reconstruction. Rerunning reconstruction..."
                    )
                # check for bug where CK resolution is 0.0
                if float(finalCKresErr) == 0.0:
                    issue_present = True
                    print(
                        "CK resolution error is 0.0. Rerunning reconstruction..."
                    )

                if issue_present:
                    return issue_present

    return issue_present


def write_plotting(ck, x, ckerr, csv_name):

    conditions = []

    for axis in x.keys():
        if x[axis] is not None:
            conditions.append(f'{axis}{x[axis]}')

    conds_to_write = ':'.join(str(d) for d in conditions)

    with open(f'ck_vs_param_R{whichRich}_{survey_type}_{csv_name}.csv',
              'a') as f:
        writer = csv.writer(f)
        writer.writerow([ck, conds_to_write, ckerr])


def monitor(x, panel):
    '''
    Function to be minimised
    x - tuple of alignment parameters as:
        (panel0_x, panel1_x, panel0_y, panel1_y, panel0_z, panel1_z)
    panel - str of panel to shift, choose from '0x', '1x', '0y', '1y'

    returns resolution (need to minimise)
    '''
    # create starting dict of 0.0 for each condition
    param_list = ['p0_x', 'p1_x', 'p0_y', 'p1_y', 'p0_z', 'p1_z']
    param_dict = dict.fromkeys(param_list)
    for p_axis in param_dict.keys():
        param_dict[p_axis] = 0.0

    # set optimum alignment conditions
    if whichRich == 1:
        if survey_type == "translation":
            if whichPanel == 0:
                param_dict[f'p0_x'] = -6.0
            else:
                param_dict["p1_x"] = -4.5
                param_dict["p1_y"] = 9.0
        elif survey_type == "rotation":
            pass
    elif whichRich == 2:
        if survey_type == "translation":
            if whichPanel == 0:
                param_dict["p0_x"] = 1.5
                param_dict["p0_y"] = 1.5
            else:
                param_dict["p1_x"] = 10.5
                param_dict["p1_y"] = 3.0
        elif survey_type == "rotation":
            pass

    for axis in ["x", "y", "z"]:
        if x[axis] is not None:  # we want to update this condition
            param_dict[f'p{whichPanel}_{axis}'] = x[axis]

    print(param_dict)
    write_to_csv(param_dict)

    check_if_already_run_res = check_if_already_run(param_dict,
                                                    f'panel{panel}')
    if check_if_already_run_res:
        return

    check_for_reco_issues_res = True
    while check_for_reco_issues_res:
        result = subprocess.run(
            ['sh', f'{prepath}/run_steering_Rich{whichRich}_p{whichPanel}.sh'],
            text=True)
        alignMonitor = AlignMonitor(
            whichRich=whichRich,
            floated=[
                param_dict[f'p{whichPanel}_x'], param_dict[f'p{whichPanel}_y'],
                param_dict[f'p{whichPanel}_z']
            ])
        alignMonitor.performMonitoring(panel)
        finalCKres = alignMonitor.finalCKres
        finalCKresErr = alignMonitor.finalCKresErr
        check_for_reco_issues_res = check_for_reco_issues(
            finalCKres, finalCKresErr, f'panel{panel}')
        # sys.stdout.flush()
        os.remove(f'rich{whichRich}_opt_dd4hep_p{whichPanel}.root')
        os.remove(f'rich{whichRich}_opt_dd4hep_p{whichPanel}_new.root')

    os.remove(f'rich{whichRich}_{survey_type}_params_p{whichPanel}.csv')
    write_plotting(finalCKres, param_dict, finalCKresErr, f'panel{panel}')
    return finalCKres


def minimizer(p_axis, iter_num, step_size):

    split_p_axis = list(map(str, p_axis))
    p = split_p_axis[0]
    axes = split_p_axis[1:]
    conds_update = {"x": None, "y": None, "z": None}
    init_conds = {"x": None, "y": None, "z": None}

    if whichRich == 1 and whichPanel == 0:
        # # move in +y direction
        # init_dict[f'R{whichRich}'][f'{p}y'][f'{survey_type}'] = -7.5 + 3*step_size
        # # move in +x direction
        # init_dict[f'R{whichRich}'][f'{p}x'][f'{survey_type}'] = -1.5 + 2*step_size

        # increase granularity in certain region
        init_dict[f'R{whichRich}'][f'{p}x'][f'{survey_type}'] = 0.0 - (
            step_size) - (step_size / 2)
        init_dict[f'R{whichRich}'][f'{p}y'][f'{survey_type}'] = -7.5 + (
            2 * step_size) + (step_size / 2)

        num_iter = {}
        num_iter["x"] = 5
        num_iter["y"] = 6

        manual_points = {
            "x": [
                0.0 + (step_size), 0.0 + (2 * step_size),
                0.0 + (step_size) + (step_size / 2),
                0.0 + (step_size) + (step_size / 2)
            ],
            "y": [
                -4.5 - step_size - step_size / 2,
                -4.5 - step_size - step_size / 2, -4.5 - step_size,
                -4.5 - 2 * step_size
            ]
        }

    if whichRich == 1 and whichPanel == 1:
        # extend +x and -y

        # increase granularity in certain region
        init_dict[f'R{whichRich}'][f'{p}x'][f'{survey_type}'] = -4.5 - (
            2 * step_size) - (step_size / 2)
        init_dict[f'R{whichRich}'][f'{p}y'][f'{survey_type}'] = 7.5 + (
            step_size / 2)

        num_iter = {}
        num_iter["x"] = 8
        num_iter["y"] = 8

    if whichRich == 2 and whichPanel == 0:
        # extend -x
        # init_dict[f'R{whichRich}'][f'{p}x'][f'{survey_type}'] = 0.0 - 4*step_size
        # # extend -y : change this
        # init_dict[f'R{whichRich}'][f'{p}y'][f'{survey_type}'] = 0.0 - 2*step_size

        # increase granularity in certain region
        init_dict[f'R{whichRich}'][f'{p}x'][f'{survey_type}'] = 0.0 - (
            step_size) - (step_size / 2)
        init_dict[f'R{whichRich}'][f'{p}y'][f'{survey_type}'] = 0.0 + (
            step_size) + (step_size / 2)

        num_iter = {}
        num_iter["x"] = 8
        num_iter["y"] = 4

        manual_points = {
            "x": [
                0.0 + (step_size / 2), 0.0 + (2 * step_size),
                0.0 + (2 * step_size), 1.5 + (step_size) + (step_size / 2),
                1.5 + (2 * step_size) + (step_size / 2), 1.5
            ],
            "y": [
                1.5, 1.5 - step_size, 1.5 - (2 * step_size), 1.5, 1.5,
                1.5 - (step_size / 2)
            ]
        }

    if whichRich == 2 and whichPanel == 1:
        # # extend -x
        # init_dict[f'R{whichRich}'][f'{p}x'][f'{survey_type}'] = 9.0 -4*step_size
        # # extend -y
        # init_dict[f'R{whichRich}'][f'{p}y'][f'{survey_type}'] = 1.5 -2*step_size

        # # extend +x
        # init_dict[f'R{whichRich}'][f'{p}x'][f'{survey_type}'] = 9.0 + 2*step_size

        # increase granularity in certain region
        init_dict[f'R{whichRich}'][f'{p}x'][f'{survey_type}'] = 9.0 - (
            step_size / 2)
        init_dict[f'R{whichRich}'][f'{p}y'][f'{survey_type}'] = 1.5 + (
            step_size / 2)

        num_iter = {}
        num_iter["x"] = 7
        num_iter["y"] = 6

        manual_points = {
            "x": [
                9.0,
                9.0 + (2 * step_size),
                9.0 + (3 * step_size),
                9.0 + (step_size) + (3 / 4 * step_size),
            ],
            "y": [
                1.5 + (2 * step_size) + (step_size / 2), 3.0 + (step_size / 2),
                3.0 + (step_size / 2), 3.0 + (step_size) + (step_size / 4)
            ]
        }

    for axis in axes:
        conds_update[axis] = init_dict[f'R{whichRich}'][f'{p}{axis}'][
            f'{survey_type}']
        init_conds[axis] = init_dict[f'R{whichRich}'][f'{p}{axis}'][
            f'{survey_type}']

    if num_chunks != 0:

        # this if statement has not been updated yet
        this_iter_chunk_list = get_chunk_list(x0)
        for x0_i in this_iter_chunk_list:
            res = monitor(x0_i, p)

    else:

        if twoD_scan is False:
            for i in range(iter_num):
                conds_update[axes[0]] += step_size
                res = monitor(conds_update, p)
        else:

            if run_manual_points:
                for i in range(0, len(manual_points["x"])):
                    conds_update[axes[0]] = manual_points["x"][i]
                    conds_update[axes[1]] = manual_points["y"][i]

                    res = monitor(conds_update, p)
            else:
                for i_x in range(num_iter["x"] + 1):
                    if i_x != 0:
                        conds_update[axes[0]] += step_size
                    conds_update[axes[1]] = init_conds[axes[1]]

                    for i_y in range(num_iter["y"] + 1):
                        if i_y != 0:
                            conds_update[axes[1]] += step_size

                        res = monitor(conds_update, p)


def get_chunk_list(x0):

    iter_list = [x0]
    update_x0 = x0[0]
    # obtain list of conditions to iterate over
    for i in range(iter_num):
        update_x0 += step_size
        iter_list.append([update_x0])

    iter_chunk_list = np.array_split(iter_list, num_chunks)
    this_iter_chunk_list = iter_chunk_list[chunk_number]

    return this_iter_chunk_list


# can minimise using monitor() as function to minimise if returns finalCKres and has arg inputs
# Optimal params (DetDesc MC):
# RICH1: [0.0, 0.0, -30.15, 30.15, 0.0, 0.0]
# RICH2: [-1.32, 1.41, -4.74, -3.92, 4.79, 2.57]

if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description="Run optimisation on RICH panel alignment")
    parser.add_argument(
        '--whichRich',
        type=int,
        choices=[1, 2],
        help='Which RICH panel to optimise')

    parser.add_argument(
        '--whichPanel',
        type=str,
        choices=['both', '0', '1'],
        help='Which Panel panel to use')

    parser.add_argument(
        '--survey_type',
        type=str,
        help='Which tests to run',
        choices=["translation", "rotation"])

    parser.add_argument(
        '--twoD_scan',
        type=str,
        help='Perform 2D or 1D scans',
        default='False')

    # arguments for batch submission
    parser.add_argument(
        '--axis',
        type=str,
        help='Which axis to translate along or rotate about',
        choices=["x", "y", "z"])

    parser.add_argument(
        '--num_chunks',
        type=int,
        default=0,
        help=
        'Number of chunks to split up survey into (each chunk runs on a separate node)'
    )

    parser.add_argument('--chunk_number', type=int, help='Chunk identifier')

    args = parser.parse_args()
    global whichRich
    whichRich = int(args.whichRich)
    global whichPanel
    whichPanel = int(args.whichPanel)
    global survey_type
    survey_type = args.survey_type
    global twoD_scan
    twoD_scan = json.loads(args.twoD_scan.lower())

    # arguments for batch submission
    global axis
    axis = args.axis
    global num_chunks
    num_chunks = args.num_chunks
    global chunk_number
    chunk_number = args.chunk_number

    prepath = '/eos/user/j/jreich/stack_dd4hep_december/Panoptes/Rich/RichMirrorAlignmentOnline/files/OfflineScripts/PanelAlignment/surveys'

    if survey_type == "translation":
        # iter_num = 60
        # step_size = 0.25

        # iter_num = 30
        # step_size = 0.5

        iter_num = 8
        step_size = 0.375  #(in mm)

        offset = int(iter_num / 2) * step_size

        run_manual_points = True

    elif survey_type == "rotation":
        iter_num = 60
        step_size = 0.85

    # define starting conditons of each survey
    global init_dict
    init_dict = {
        "R1": {
            "0x": {
                "translation": 0.0 - offset,
                "rotation": -25.0
            },
            "0y": {
                "translation": -6.0 - offset,
                "rotation": -25.0
            },
            "0z": {
                "translation": 0.0,
                "rotation": -25.0
            },
            "1x": {
                "translation": -4.5 - offset,
                "rotation": -25.0
            },
            "1y": {
                "translation": 9.0 - offset,
                "rotation": -25.0
            },
            "1z": {
                "translation": 0.0,
                "rotation": -25.0
            },
        },
        "R2": {
            "0x": {
                "translation": 1.5 - offset,
                "rotation": -25.0
            },
            "0y": {
                "translation": 1.5 - offset,
                "rotation": -25.0
            },
            "0z": {
                "translation": 0.0,
                "rotation": -25.0
            },
            "1x": {
                "translation": 10.5 - offset,
                "rotation": -25.0
            },
            "1y": {
                "translation": 3.0 - offset,
                "rotation": -25.0
            },
            "1z": {
                "translation": 0.0,
                "rotation": -25.0
            },
        }
    }

    if num_chunks != 0:
        minimizer(f'{whichPanel}{axis}')
    else:
        minimizer(f'{whichPanel}xy', iter_num, step_size)
