###############################################################################
# (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
__author__ = "Claire Prouve"
__copyright__ = "University of Bristol, November 2022"

import time
from time import sleep
from time import time
from time import gmtime, strftime
import pydim
import threading
import shutil
import os
import sys


class SaveSetRetriever:
    def __init__(self, part, tsk):
        self.name = part + "/" + tsk + "/SAVESETLOCATION"
        self.format = "C"
        self.SaveSet = ""
        self.lastData = 0.0
        self.__lock = threading.Lock()
        self.__lock.acquire()
        self.__infoID = pydim.dic_info_service(
            self.name,
            self.format,
            self.callback,
            timeout=10000000,
            default_value="Default")
        self.__lock.release()
        print("INFO: Constructed SaveSet Retriever using service name " +
              self.name)

    def callback(self, *args):
        self.__lock.acquire()
        print("INFO: SaveSetRetriever Callback Called with args ", args)
        if len(args) > 1:
            r = args[1]
            r = r[:r.find('\0')]
            self.SaveSet = r
            self.lastData = time()
        self.__lock.release()
        sys.stdout.flush()

    def getSaveSet(self):
        return self.SaveSet

    def getUpdTime(self):
        return self.lastData


class HistoCopier:
    def __init__(self, alignConf, ssretr):
        self.__tiltNames = alignConf.getProp('tiltNames')
        self.__workdir = alignConf.getProp('WorkDir')
        self.__whichRich = alignConf.getProp('Rich')
        self.__nameStr = alignConf.getProp('nameStr')
        self.__ssretr = ssretr

    def resultName(self, n_it, m_it):
        if m_it % 9 == 0:
            mergename = (self.__workdir + "/RichRecQCHistos_rich" + str(
                self.__whichRich) + "_" + self.__nameStr + "_i" + str(n_it) +
                         ".root")
        else:
            mergename = (
                self.__workdir + "/RichRecQCHistos_rich" + str(
                    self.__whichRich) + "_" + self.__nameStr + "_" +
                self.__tiltNames[m_it % 9] + "_i" + str(n_it) + ".root")
        return mergename

    def waitForSaveSet(self, resultName, m_it):
        waitForSs = True
        while (waitForSs):
            SaveSetName = self.__ssretr.getSaveSet()
            if str('00' + str(m_it)) in SaveSetName:
                waitForSs = False
            sleep(1)
        waitForSs = True
        while (waitForSs):
            if os.path.exists(SaveSetName):
                waitForSs = False
            sleep(1)
        shutil.copyfile(SaveSetName, resultName)

    def getHistoFile(self, n_it, m_it):
        resultName = self.resultName(n_it, m_it)
        self.waitForSaveSet(resultName, m_it)
