###############################################################################
# (c) Copyright 2022 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
__authors__ = "Claire Prouve, Paras Naik"
__copyright__ = "University of Bristol, November 2022"


#
# THIS IS THE FLATTENED ITERATOR
# IT ONLY HAS ONE JOB
# RUN MIRRCOMBINFIT AND RUN MIRRALIGN
#
# PROPAGATE ANY USEFUL CHANGES TO THE MAIN ITERATOR
#
# A commented line that starts with #$# exists in the main iterator
#  but was commented specifically for this flattened iterator
#
# YOU MUST PUT THE ROOT FILE IN THE RIGHT PLACE!!!
class State(object):
    UNKNOWN = "UNKNOWN"
    ERROR = "ERROR"
    NOT_READY = "NOT_READY"
    READY = "READY"
    RUNNING = "RUNNING"
    PAUSED = "PAUSED"


#$# from Communicator import *  # gets the State and Communicator classes
import distutils
import os
import re
import shutil
import sys
import math
import time
from time import sleep
from time import time
from time import gmtime, strftime
import threading
import Configurables
from subprocess import *
from distutils import dir_util
from ruamel.yaml import YAML

print("INFO: Iterator_flat.py launched at " +
      strftime("%Y-%m-%d %H:%M:%S", gmtime()) + " UTC")
sys.stdout.flush()


########################################################################################################################
##### Here is where the party starts!
########################################################################################################################
def run(whichRich):
    launchTime = strftime("%Y-%m-%d %H:%M:%S", gmtime())
    start_time_config = time()  #  when LHCbA config start time is really set

    ### Set up strings needed for logging of hlt02 (the node that the iterator is running on)
    loggingDir = '/group/rich/AlignmentFiles/Logging/'
    lastAlignSuccess = loggingDir + 'Rich' + str(
        whichRich) + '_' + 'success.txt'
    currentHLT02log = loggingDir + 'Rich' + str(whichRich) + '_' + 'hlt02.log'
    currentHLT02logReduced = loggingDir + 'Rich' + str(
        whichRich) + '_' + 'hlt02reduced.log'
    previousHLT02log = loggingDir + 'Rich' + str(
        whichRich) + '_' + 'hlt02_previous.log'
    previousHLT02logReduced = loggingDir + 'Rich' + str(
        whichRich) + '_' + 'hlt02reduced_previous.log'
    cmdStart = 'exec /opt/FMC/bin/logViewer -l 2 -S -b &>'
    cmdProcess = cmdStart + ' ' + currentHLT02log
    cmdProcessAppend = cmdStart + '> ' + currentHLT02log

    # string used to help remove lines from the hlt02 log in the hlt02reduced versions,
    #   using grep -vE (E = Enhanced Mode, required to parse the string)
    removeExtraneous = '(LHCb2_HLT02_Hlt2Adder_0|Hlt2SaverSvc|MARK|BusyPub|BusyMon|JobOptionsSvc|environtment)'  # environtment not a typo

    ## Backup the last HLT02 log
    if os.path.exists(currentHLT02log):
        shutil.copyfile(currentHLT02log, previousHLT02log)
    if os.path.exists(previousHLT02log):
        os.system('grep -vE "' + removeExtraneous + '" ' + previousHLT02log +
                  ' > ' + previousHLT02logReduced)

    ### Check to see if the last mirror alignment was successful
    if os.path.exists(lastAlignSuccess):
        with open(lastAlignSuccess) as lastAlignSuccessFile:
            lastAlignSuccessLine = lastAlignSuccessFile.readline()
    else:
        lastAlignSuccessLine = 'UNKNOWN\n'
    ### Start logging of hlt02 (the node that the iterator is running on)
    if lastAlignSuccessLine == 'YES\n':
        logProcess = Popen(
            cmdProcess,
            shell=True,
            executable="/bin/bash",
            stderr=PIPE,
            stdout=PIPE)
    else:
        logProcess = Popen(
            cmdProcessAppend,
            shell=True,
            executable="/bin/bash",
            stderr=PIPE,
            stdout=PIPE)
    logProcess.stdout.close()
    logProcess.stderr.close()
    sys.stdout.flush()
    sleep(8)  # wait long enough for logger to initialize

    ### Reset lastAlignSuccessFile to default
    lastAlignSuccessFile = open(lastAlignSuccess, 'w')
    lastAlignSuccessFile.write('NO\n')  # python will convert \n to os.linesep
    lastAlignSuccessFile.close()

    ### Printout (slightly late, so it gets logged)
    print("INFO: run(whichRich = " + str(whichRich) +
          ") command received at " + launchTime + " UTC")
    print("INFO: LHCbA config start time set.")
    print("INFO: The launch directory is: ", os.getcwd())
    if lastAlignSuccessLine == 'YES\n':
        print(
            "INFO: The previous Rich" + str(whichRich) +
            " alignment was able to copy hlt02.log, overwriting hlt02.log with this alignment's printouts."
        )
    else:
        print(
            "INFO: The previous Rich" + str(whichRich) +
            " alignment was not able to copy hlt02.log, appending this alignment's printouts to hlt02.log."
        )
    print("INFO: logging hlt02 in temporary file: " + currentHLT02log)
    print("INFO: backing up last hlt02 log in temporary file: " +
          previousHLT02log)
    print("INFO: storing reduced backup of last hlt02 log in temporary file: "
          + previousHLT02logReduced)

    ### make a backup of OnlineEnvBase.py (do it now, in case we need to study the configuration on a single node)
    saveOnlineEnvBase = loggingDir + 'Rich' + str(
        whichRich) + '_' + 'OnlineEnvBase.py'
    shutil.copyfile(
        '/group/online/dataflow/options/LHCbA/OnlineEnvBase.py',  # PN - changed from "LHCbA/HLT/OnlineEnvBase.py" where it was in Run 2
        saveOnlineEnvBase)
    print("INFO: storing OnlineEnvBase.py in temporary file: " +
          saveOnlineEnvBase)
    sys.stdout.flush()

    ### make a backup of Configuration.py
    ### PN - need to update this: (make sure AlignmentRelease(?) on /group/rich/sw/alignment/stack/(?) points to our newest AlignmentOnline release!)
    saveConfiguration = loggingDir + 'Rich' + str(
        whichRich) + '_' + 'Configuration.py'
    shutil.copyfile(
        #        '/group/rich/sw/cmtuser/AlignmentRelease/Rich/RichMirrorAlignmentOnline/python/RichMirrorAlignmentOnline/Configuration.py',
        '/group/rich/sw/alignment/stack/Panoptes/Rich/RichMirrorAlignmentOnline/python/RichMirrorAlignmentOnline/Configuration.py',
        saveConfiguration)
    print("INFO: storing Configuration.py in temporary file: " +
          saveConfiguration)
    sys.stdout.flush()

    from HistoHelper import SaveSetRetriever
    ssRetr = None  #the SaveSetRetriever, this thing will give us the path to the histogram files produced by the analyzers
    from HistoHelper import HistoCopier
    histCopier = None  #the HistoCopier, it will wait after each minor iteration for the savesets to be written and copy it under the approriate name into workdir
    from XMLFileHelper import XMLFileHelper
    xmlHelper = None
    from YAMLFileHelper import YAMLFileHelper
    ymlHelper = None
    from convert_XML_YAML import Conversion
    conversionsHelper = None
    from RichAlignmentHelper import RichAlignmentHelper
    alignHelper = None
    from SetupHelper import SetupHelper
    setupHelper = None
    from UpdateHelper import UpdateHelper
    updateHelper = None

    ########################################################################################################################
    #####  reading in the settings from the Configuration file
    ########################################################################################################################
    print("INFO: Importing Configuration")
    sys.stdout.flush()
    if whichRich == 1:
        #$# from Configurables import Rich1MirrAlignOnConf
        from Configuration import Rich1MirrAlignOnConf
        alignConf = Rich1MirrAlignOnConf()
    elif whichRich == 2:
        #$# from Configurables import Rich2MirrAlignOnConf
        from Configuration import Rich2MirrAlignOnConf
        alignConf = Rich2MirrAlignOnConf()
    else:
        sys.stderr.write("A detector named RICH" + str(whichRich) +
                         " does not exist.\n")
        sys.exit(1)
    alignConf.__apply_configuration__()
    print("INFO: Configuration Applied")
    sys.stdout.flush()

    maxIters = int(
        alignConf.getProp('maxIterations')
    )  # This is the maximum number of iterations before the alignment gives up
    workdir = alignConf.getProp('WorkDir')
    #histdir = alignConf.getProp('HistoDir') - this is never used!
    tiltNames = alignConf.getProp('tiltNames')
    tiltNamesLength = len(tiltNames)
    thisNameStr = alignConf.getProp('nameStr')
    magnFactorsMode = alignConf.getProp('magnFactorsMode')
    autoUpdate_initial = bool(alignConf.getProp('autoUpdate'))
    print("INFO: Initial setting of autoUpdate = " + str(autoUpdate_initial))
    autoUpdateNoV = False  # this can never be True if autoUpdate_initial is False
    autoUpdate = False
    if autoUpdate_initial:
        autoUpdate = True
        autoUpdateNoV = bool(alignConf.getProp('autoUpdateNoV'))
        print("INFO: Initial setting of autoUpdateNoV = " + str(autoUpdateNoV))
    testing = bool(alignConf.getProp('testing'))
    print("INFO: Initial setting of testing = " + str(testing))
    YAMLio = bool(alignConf.getProp('YAMLio'))
    if YAMLio:
        print(
            "INFO: Expecting the (v-numbered) starting file to be YAML. Convert to XML for RichAnalyzer if useDD4hep = False. Output YAML."
        )
    else:
        print(
            "INFO: Expecting the (v-numbered) starting file to be XML. Convert to YAML for MirrCombinFit & MirrAlign. Output YAML."
        )
    useDD4hep = bool(alignConf.getProp('useDD4hep'))
    if useDD4hep:
        YAMLio = True  # if using DD4hep we aren't expecting XML anymore
        print(
            "INFO: Reconstructing with DD4hep. Forced YAMLio = True. Expect RICH calibrations to be in calib.yml (or Run3 equivalent)."
        )
    else:
        print(
            "INFO: Reconstructing with DetDesc. Expect RICH calibrations to be in calib.xml"
        )
    """
    # This is where the combAndMirrSubsets file will be stored in the workdir.
    # We copy it into this file later, from alignConf.getProp('combAndMirrSubsets')
    # combAndMirrSubsets = workdir + "Rich" + str(whichRich) + "CombAndMirrSubsets.txt"
    combAndMirrSubsets = workdir + os.path.basename(
        alignConf.getProp('combAndMirrSubsets')
    )
    """

    ### this is the place where the analyzers will pick up the yml (xml) file for their DD4hep (DetDesc) Moore jobs
    ### before each new minor iteration the new yml (xml) file will be copied here
    currentDBpre = workdir + "CondDB_Rich" + str(whichRich)
    if useDD4hep:
        currentDB = currentDBpre + ".yml"
    else:
        currentDB = currentDBpre + ".xml"

    ### this is the place where the final output yml (xml) will be expected to be placed at the end of the alignment
    if YAMLio:
        lastDB = currentDBpre + ".yml"
    else:
        lastDB = currentDBpre + ".xml"

    ### path in the work directory of the database-file that the final database will be compared to in the AlignMonitor
    compareDB = workdir + 'Rich' + str(whichRich)
    if YAMLio:
        compareDB = compareDB + '_compareYAMLFile.yml'
    else:
        compareDB = compareDB + '_compareXMLFile.xml'

    # We do this before a "configure" because we need run and directoryTime info for the next part
    setupHelper = SetupHelper(alignConf, flat=True)

    # Do not run the mirror alignment if:
    #   Any run's polarity is OFF
    #   The polarities for the runs chosen are mixed
    #   Any of the runs' RICH calibrations are missing
    runs = []  # can remove this when everything works for Run 3
    #$# import OnlineEnv as Online
    #$# runs = Online.DeferredRuns
    polarities = []
    fills = []
    for run in runs:
        polarities.append(setupHelper.getPolarity(run))
        fills.append(setupHelper.getFillNumber(run))
    print("INFO: runs = " + str(runs))
    print("INFO: fills = " + str(fills))
    polsStr = [str(item) for item in polarities]
    print("INFO: polarities = " +
          str(polsStr))  # str() gets rid of unicode flag preceding each string
    cancel = False
    preSubject, preText, postSubject, postText = [
        ''
    ] * 4  # this is a nice variable defining strategy, but note it does not work on mutable elements (such as lists)!
    polarityMiss = []
    calibMissing = []
    polarityDiff = []
    #$# for i in range(0, len(runs)):
    #$#     calibLoc = "/group/online/hlt/conditions/LHCb/NoYear/" + str(
    #$#         int(math.floor(int(runs[i]) / 1000.0))) + "/" + str(
    #$#             runs[i]) + "/rich" + str(whichRich)
    #$#     if useDD4hep:
    #$#         calibLoc = calibLoc + "calib.yml"  # PN - find out where this is for Run 3
    #$#     else:
    #$#         calibLoc = calibLoc + "calib.xml"
    #$#     if not os.path.exists(calibLoc):
    #$#         print("WARNING: Missing RICH calibration detected: " + calibLoc)
    #$#         calibMissing.append(runs[i])
    #$#     if (polarities[i] == 'OFF') or (polarities[i] != polarities[0]):
    #$#         polarityDiff.append(runs[i])
    #$#     if (polarities[i] is None):
    #$#         polarityMiss.append(runs[i])
    if polarityDiff and calibMissing:
        cancel = True
        cancelSubject = 'TEST [' + alignConf.getProp(
            'dataVariant') + ']: ' if testing else ''
        cancelSubject += 'WARNING: RICH' + str(
            whichRich) + ' mirror alignment ' + setupHelper.directoryTime
        cancelSubject += 'started, but has been cancelled (polarityDiff and calibMissing).'
        cancelText = 'RICH' + str(
            whichRich) + ' mirror alignment ' + setupHelper.directoryTime
        cancelText += 'started, but has been cancelled. \n '
        cancelText += 'Data for these runs contain magnet OFF data *or* are of mixed polarity: ' + str(
            polarityDiff) + ' \n '
        cancelText += 'Data for these runs do not yet have RICH calibrations available: ' + str(
            calibMissing) + ' \n \n '
        cancelText += 'List of all selected Runs: ' + str(
            runs) + ' \n Fills: ' + str(fills) + ' \n Polarities: ' + str(
                polsStr)
        setupHelper.sendEmail(
            cancelSubject, cancelText,
            setupHelper.emails if not testing else setupHelper.experts)
        print("WARNING: Mirror alignment will be skipped.")
        print(
            "WARNING: These runs contain magnet OFF data *or* data for these runs is of mixed polarity: "
            + str(polarityDiff))
        print(
            "WARNING: These runs do not yet have RICH calibrations available: "
            + str(calibMissing))
        messageID = setupHelper.quickWriteInLogbook(cancelSubject, cancelText)
    # Mismatched polarity check
    elif polarityDiff:
        cancel = True
        cancelSubject = 'TEST [' + alignConf.getProp(
            'dataVariant') + ']: ' if testing else ''
        cancelSubject += 'WARNING: RICH' + str(
            whichRich) + ' mirror alignment ' + setupHelper.directoryTime
        cancelSubject += 'started, but has been cancelled (polarityDiff).'
        cancelText = 'RICH' + str(
            whichRich) + ' mirror alignment ' + setupHelper.directoryTime
        cancelText += 'started, but has been cancelled. \n '
        cancelText += 'Data for these runs contain magnet OFF data *or* are of mixed polarity: ' + str(
            polarityDiff) + ' \n \n '
        cancelText += 'List of all selected Runs: ' + str(
            runs) + ' \n Fills: ' + str(fills) + ' \n Polarities: ' + str(
                polsStr)
        setupHelper.sendEmail(
            cancelSubject, cancelText,
            setupHelper.emails if not testing else setupHelper.experts)
        print("WARNING: Mirror alignment will be skipped.")
        print(
            "WARNING: These runs contain magnet OFF data *or* data for these runs is of mixed polarity: "
            + str(polarityDiff))
        messageID = setupHelper.quickWriteInLogbook(cancelSubject, cancelText)
    # RICH calibrations availability check
    elif calibMissing:
        cancel = True
        cancelSubject = 'TEST [' + alignConf.getProp(
            'dataVariant') + ']: ' if testing else ''
        cancelSubject += 'WARNING: RICH' + str(
            whichRich) + ' mirror alignment ' + setupHelper.directoryTime
        cancelSubject += ' started, but has been cancelled (calibMissing).'
        cancelText = 'RICH' + str(
            whichRich) + ' mirror alignment ' + setupHelper.directoryTime
        cancelText += 'started, but has been cancelled. \n '
        cancelText += 'Data for these runs do not yet have RICH calibrations available: ' + str(
            calibMissing) + ' \n \n '
        cancelText += 'List of all selected Runs: ' + str(
            runs) + ' \n Fills: ' + str(fills) + ' \n Polarities: ' + str(
                polsStr)
        setupHelper.sendEmail(
            cancelSubject, cancelText,
            setupHelper.emails if not testing else setupHelper.experts)
        print("WARNING: Mirror alignment will be skipped.")
        print(
            "WARNING: These runs do not yet have RICH calibrations available: "
            + str(calibMissing))
        messageID = setupHelper.quickWriteInLogbook(cancelSubject, cancelText)
    # Missing polarity info
    elif polarityMiss:
        preSubject += 'WARNING: Some polarity information was missing, but '
        preText += 'Polarity information for these runs is currently not found: ' + str(
            polarityMiss) + ' \n'
        print(
            "WARNING: Polarity information for these runs is currently not found: "
            + str(polarityMiss) + ". This alignment will continue as normal.")
    polarityDir = '/group/rich/AlignmentFiles/Polarity/'
    forcedUpdate_ChMagPol = False
    ChMagPolSubject = ''
    ChMagPolText = ''
    polarities.append('UP')  # for Iterator_flat only
    if (not cancel) and (polarities[0] is not None):
        oldPolarityFilename = polarityDir + 'previousPolarity_RICH' + str(
            whichRich) + '.txt'
        newPolarityFilename = polarityDir + 'currentPolarity_RICH' + str(
            whichRich) + '.txt'
        with open(newPolarityFilename, 'w') as currentAlignPolarityFile:
            currentAlignPolarityFile.write(polarities[0] + '\n' + str(runs) +
                                           '\n' + str(fills) + '\n')
        if os.path.exists(oldPolarityFilename):
            with open(oldPolarityFilename, 'r') as previousAlignPolarityFile:
                first_line = previousAlignPolarityFile.readline()
            if (first_line != polarities[0] + '\n'):
                ChMagPolEmailSubject = 'TEST [' + alignConf.getProp(
                    'dataVariant') + ']: ' if testing else ''
                ChMagPolEmailSubject += 'WARNING: MAGNET POLARITY CHANGE! RICH' + str(
                    whichRich
                ) + ' mirror alignment ' + setupHelper.directoryTime + '.'
                ChMagPolEmailText = 'WARNING: MAGNET POLARITY CHANGE from \n ' + first_line + ' to ' + polarities[
                    0] + ' \n RICH' + str(
                        whichRich
                    ) + ' mirror alignment ' + setupHelper.directoryTime + '. \n Runs: ' + str(
                        runs
                    ) + ' \n Fills: ' + str(fills) + ' \n Polarities: ' + str(
                        polsStr) + ' \n dataVariant: ' + alignConf.getProp(
                            'dataVariant')
                setupHelper.sendEmail(
                    ChMagPolEmailSubject, ChMagPolEmailText,
                    setupHelper.emails if not testing else setupHelper.experts)
                forcedUpdate_ChMagPol = True
                ChMagPolSubject += 'Magnet Polarity Changed; possibly forcing update.'
                ChMagPolText += 'If this alignment converges and passes sanity checks, the alignment will be updated due to a change in magnet polarity (unless autoUpdate is/becomes False, or testing is True). If there are any problems with this alignment, please contact the Mirror Alignment expert! They should check previousPolarity_RICH in ' + polarityDir + ' to ensure the proper behavior on the next attempt!'
                postSubject += ' ' + ChMagPolSubject
                postText += ' \n ' + ChMagPolText
                print(
                    "INFO: Magnet Polarity Changed; if this alignment converges under normal conditions, then the alignment will be updated."
                )
                # Next two lines only get used at the end if things fail
                ChMagPolSubject = ' WARNING: The magnet polarity changed!'
                ChMagPolText = ' \n WARNING: The magnet polarity changed! Make sure that everything (e.g. previousPolarity_RICH) is in order for the next alignment!!!'
                # OUTDATED: The expert should re-flip the magnet polarity for this Rich detector in ' + polarityDir + ' in order to have the proper behavior on the next attempt!!!

    if (not cancel):
        leadText = ''
        if testing:
            leadText += 'TEST [' + alignConf.getProp('dataVariant') + ']: '
        totalSubject = leadText + preSubject + 'RICH' + str(
            whichRich
        ) + ' alignment ' + setupHelper.directoryTime + ' has started.' + postSubject
        totalText = preText + ' RICH' + str(
            whichRich
        ) + ' mirror alignment ' + setupHelper.directoryTime + ' has started. \n Runs: ' + str(
            runs) + ' \n Fills: ' + str(fills) + ' \n Polarities: ' + str(
                polsStr) + postText + ' \n dataVariant: ' + alignConf.getProp(
                    'dataVariant')
        # setupHelper.sendEmail( totalSubject , totalText, setupHelper.emails if not testing else setupHelper.experts)
        postText += " \n \n If this alignment does not finish, please report this via the RICH ELOG. \n "
        totalText = preText + ' RICH' + str(
            whichRich
        ) + ' mirror alignment ' + setupHelper.directoryTime + ' has started. \n Runs: ' + str(
            runs) + ' \n Fills: ' + str(fills) + ' \n Polarities: ' + str(
                polsStr) + postText + ' \n dataVariant: ' + alignConf.getProp(
                    'dataVariant')
        print("INFO: RICH" + str(whichRich) + " mirror alignment " +
              setupHelper.directoryTime + " has started, dataVariant: " +
              alignConf.getProp('dataVariant'))
        messageID = setupHelper.quickWriteInLogbook(totalSubject, totalText)

    sys.stdout.flush()

    ########################################################################################################################
    ##### define the communicator who will communicate with the run-control
    ########################################################################################################################

    state = State.NOT_READY
    #$# com = Communicator('ALIGNITER')
    print("INFO: Communicator started.")
    #$# com.set_status(state)
    print("INFO: Iterator_flat state = " + str(state))

    conv = False  ### variable indicates if alignment converged yet

    while True:
        sys.stdout.flush()  # Always keep this line in
        print("INFO: Waiting for command from Communicator")
        sys.stdout.flush()
        if not cancel:
            #$#     command = com.get_command()
            acceptable_commands = [
                "configure", "start", "pause", "reset", "stop", "c", "s", "p",
                "r", "t"
            ]
            command = None
            while command is None:
                command = input(
                    "You are the Communicator. Enter command (configure [or c], start [or s], pause [or p], reset [or r], stop [or t]): "
                )
                print("You have provided the following command: " + command)
                if command not in acceptable_commands:
                    print("Please try again.")
                    command = None
            if command == "c":
                command = "configure"
            if command == "s":
                command = "start"
            if command == "p":
                command = "pause"
            if command == "r":
                command = "reset"
            if command == "t":
                command = "stop"
        else:
            command = 'reset'
        ########################################################################################################################
        ##### this code will be executed when you click configure on the run-control
        ########################################################################################################################
        if command.startswith('configure') and state == State.NOT_READY:
            print("INFO: CONFIGURE command received at " +
                  strftime("%Y-%m-%d %H:%M:%S", gmtime()) + " UTC")
            sys.stdout.flush()
            xmlHelper = XMLFileHelper(alignConf)
            ymlHelper = YAMLFileHelper(alignConf)
            conversionsHelper = Conversion()
            #$# ssretr = SaveSetRetriever("LHCbA", "AligWrk_Rich" + str(whichRich))
            #$# histCopier = HistoCopier(alignConf, ssretr)
            #alignHelper = RichAlignmentHelper(alignConf, combAndMirrSubsets)
            alignHelper = RichAlignmentHelper(alignConf)
            updateHelper = UpdateHelper(alignConf)
            ### start the Mirror Alignment DB update DIM server for this RICH detector
            if not testing:
                updateHelper.startUpdateDIM()
            else:
                print(
                    "INFO: The alignment is in testing mode. DB update DIM service disabled."
                )
            sys.stdout.flush()
            ### get the name of the yml (xml) file that is equivalent to "the DB alignment" within the '/group/online/alignment/' directory on plus
            if YAMLio:
                latestName = ymlHelper.getStartYAMLOrigFileName()
                latest_vN = (latestName.replace(".yml", "")).replace("v", "")
            else:
                latestName = xmlHelper.getStartXMLOrigFileName()
                latest_vN = (latestName.replace(".xml", "")).replace("v", "")
            state = State.READY  # Though LHCbA is still configuring
            sys.stdout.flush()

        ########################################################################################################################
        ##### setting up the alignment and starting the first minor iteration (this only for the very first iteration)
        ########################################################################################################################
        elif (command.startswith('start') and state == State.READY):
            print("INFO: START command received at " +
                  strftime("%Y-%m-%d %H:%M:%S", gmtime()) + " UTC")
            sys.stdout.flush()

            ########################################################################################################################
            ##### iteration counters, n_it for major iteration, m_it for minor iteration
            ########################################################################################################################
            n_it = alignConf.getProp("MajItStart")
            m_it = alignConf.getProp("MinItStart")
            if (n_it != 0) or (m_it != 0):
                print("INFO: n_it, m_it =", n_it, ",", m_it)

            try:
                elapsed_time_config
            except NameError:
                elapsed_time_config = time(
                ) - start_time_config  # LHCbA configuration time
            else:
                elapsed_time_config = 0.0  # we were already configured
                if (n_it == 0) and (m_it == 0):
                    setupHelper.directoryTime = strftime(
                        "%Y%m%d_%H%M%S", gmtime()
                    )  # create a new storage directory if we STARTed a second clean alignment from the same Iterator configuration
            print("INFO: Elapsed time to configure LHCbA set as",
                  elapsed_time_config,
                  "seconds.")  # only accurate if LHCbA AutoPilot is ON

            elapsed_runtimes = {
                'elapsed_time_run': 0.0,
                'elapsed_times_RichMirrCombinFit': [],
                'elapsed_times_RichMirrAlign': []
            }
            start_time_run = time()
            print("INFO: Alignment start time set.")
            sys.stdout.flush()

            #$# ### write the file for the analyzers to know which iteration they are at
            #$# f = open(alignConf.getProp("ItNrFile"), "w")
            #$# f.write(str(m_it))
            #$# f.close()
            t = time()

            ### if you want to start the alignment from scratch (e.g. at the m_it = 0)
            if m_it == 0:
                #clean up the working directory
                if YAMLio:
                    print(
                        "INFO: Setting up the folders and getting the starting YAML"
                    )
                else:
                    print(
                        "INFO: Setting up the folders and getting the starting XML"
                    )
                print("INFO: Removing everything in " + workdir)
                for f in os.listdir(workdir):
                    if os.path.isdir(workdir + "/" + f):
                        shutil.rmtree(workdir + "/" + f)
                    else:
                        os.remove(workdir + "/" + f)
                """
                shutil.copyfile(
                    alignConf.getProp('combAndMirrSubsets'),
                    combAndMirrSubsets)
                print("INFO: combAndMirrSubsets copied from " +
                      alignConf.getProp('combAndMirrSubsets') + " to " +
                      combAndMirrSubsets)
                """

                #consistency workflow timestamp
                yaml = YAML()
                with open(
                        workdir + "rich" + str(whichRich) +
                        "_session_timestamp.yml", "w") as fout:
                    yaml.dump(setupHelper.dt, fout)

                #consistency workflow variant
                mp = alignConf.getProp('dataVariant')
                # evt_max = 1000 # online we do not know this a priori
                # n_kevts = str(math.trunc(evt_max / 1000)) + 'k' # online we do not know this a priori

                # PN - at some point we need to
                #      feed these in from somewhere else
                #      if it needs to be changed often!
                addinfo = '_minp60_subset_' + mp  # + n_kevts
                #addinfo = '_minp60_' + mp # + n_kevts
                #addinfo = '_minp60_qnt20f60_rndmpresc_' + mp # + n_kevts
                #addinfo = ''
                #addinfo = '_' + mp + n_kevts
                #addinfo = '_minp60_phi2_' + mp # + n_kevts
                #addinfo = '_minp60_phi_' + mp # + n_kevts
                #addinfo = '_minp60_qnt20f60_' + mp # + n_kevts
                #addinfo = '_minp60_subset_theta2_' + mp # + n_kevts
                mirror_align_tasks = [
                    'Produce',  # fill the production set of histograms
                    #'Monitor',  # add various checking histograms
                    #'MapExlore',  # explore tracks and photons for the HLT1 pre-selection line "map"
                    #'MapConstruct',  # construct exploratory data structure for the HLT1 pre-selection line "map"
                    #'Map',  # create data structure to be used in the HLT1 selection line with entire contents
                    #'MapUse',  # use the "map" data structure a la in the HLT1 pre-selection line "map"
                    #'MapPhi',  # create data structure to be used in the HLT1 selection line with contents in quantiles
                    #'MapPhiUse',  # use the "mapPhi" data structure in the HLT1 pre-selection line "mapPhi"
                    #'AllFillCount',  # add counters for optimization of the RICH2 mirror combinations subset
                    #'AllFillPhi',  # fill 1D phi histos for optimization of the RICH2 mirror combinations subset
                    #'CheckRestFill',  # check filling the rest of RICH2 mirror combinations along with 8 poorest
                    #'SkipFilled',  # check filling all RICH2 mirror combinations with skipping when filled
                ]
                # cumulative substring with all tasks chosen
                tasks = ''
                for task in mirror_align_tasks:
                    tasks += '_' + task

                yaml = YAML()
                # retrieve the timestamp - PN this is completely redundant, but for the sake of consistency...
                with open(workdir + 'rich' + str(whichRich) +
                          '_session_timestamp.yml') as inp:
                    ts = yaml.load(inp)

                # form the current variant name
                current_variant = ts + '_' + 'rich' + str(
                    whichRich) + addinfo + tasks

                # record the current variant name into the YAML file
                with open(workdir + 'rich' + str(whichRich) + '_variant.yml',
                          'w') as out:
                    yaml.dump(current_variant, out)

                #currentMirrorDBFilePre = workdir + "Rich" + str(
                #    whichRich) + "CondDBUpdate_" + thisNameStr + "_i" + str(
                #        n_it)
                currentMirrorDBFilePre = workdir + current_variant + "_conds_i" + str(
                    n_it)
                if YAMLio:
                    ymlHelper.getStartYAML(
                        currentMirrorDBFilePre +
                        ".yml")  ### get the starting-yml from the database
                    #ymlHelper.getCompareYAML(
                    #    compareDB)  ### can be different from starting-yml
                    if not useDD4hep:
                        conversionsHelper.conv_YAML(currentMirrorDBFilePre +
                                                    ".yml")
                else:
                    xmlHelper.getStartXML(
                        currentMirrorDBFilePre +
                        ".xml")  ### get the starting-xml from the database
                    #xmlHelper.getCompareXML(
                    #    compareDB)  ### can be different from starting-xml
                    if not useDD4hep:
                        conversionsHelper.conv_XML(currentMirrorDBFilePre +
                                                   ".xml")

                if not magnFactorsMode == 0:
                    if useDD4hep:
                        print(
                            "INFO: magnFactorsMode is nonzero. Creating coeff_calibration_yml files."
                        )
                        ymlHelper.create_coeff_calibration_yml_files(
                            currentMirrorDBFilePre + ".yml", n_it
                        )  ### create the yml files for the tilted mirror configurations
                    else:
                        print(
                            "INFO: magnFactorsMode is nonzero. Creating coeff_calibration_xml files."
                        )
                        xmlHelper.create_coeff_calibration_xml_files(
                            currentMirrorDBFilePre + ".xml", n_it
                        )  ### create the xml files for the tilted mirror configurations
                    sys.stdout.flush()

            ### if you want to start from some arbitrary minor iteration
            else:
                #this method will keep all files needed when starting at m_it > 0
                setupHelper.startMinIt(m_it)

            #shutil.copyfile(
            #    "/group/rich/pnaik/input_Iterator_flat/Claire/rich2_variant.yml",
            #    workdir + "rich2_variant.yml")  #
            #shutil.copyfile(
            #    "/group/rich/pnaik/input_Iterator_flat/Claire/2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_conds_i3.yml",
            #    workdir +
            #    "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_conds_i3.yml"
            #)  #
            yaml = YAML()
            with open(workdir + '/' + 'rich' + str(whichRich) +
                      '_variant.yml') as finp:
                variant = yaml.load(finp)
            ### now place the yml (xml) file for the current minor iteration in the place where the analyzers will pick it up
            if m_it % tiltNamesLength == 0:
                #pickupDB = workdir + "Rich" + str(
                #    whichRich) + "CondDBUpdate_" + thisNameStr + "_i" + str(
                #        n_it)
                pickupDB = workdir + variant + "_conds_i" + str(n_it)
            else:
                #pickupDB = workdir + "Rich" + str(
                #    whichRich
                #) + "CondDBUpdate_" + thisNameStr + "_" + tiltNames[
                #    m_it % tiltNamesLength] + "_i" + str(n_it)
                pickupDB = workdir + variant + "_conds_" + tiltNames[
                    m_it % tiltNamesLength] + "_i" + str(n_it)
            if useDD4hep:
                pickupDB = pickupDB + ".yml"
            else:
                pickupDB = pickupDB + ".xml"
            shutil.copyfile(pickupDB, currentDB)

            print("INFO:    ___________________________ ")
            print("INFO:    Start new major Iteration " + str(n_it))
            print("INFO:    *************************** ")

            yaml = YAML()
            with open(workdir + 'rich' + str(whichRich) + '_iter_number.yml',
                      'w') as fout:
                yaml.dump(n_it, fout)

            print("INFO:    Minor iteration " + str(m_it))

            # increase the minor iteration counter
            if magnFactorsMode == 0:
                setupHelper.setupMagnifFiles()
                m_it = m_it + tiltNamesLength
            else:
                m_it = m_it + 1
            # set iterator state to running => analyzers will execute the Moore jobs now
            state = State.RUNNING
            sys.stdout.flush()

        ########################################################################################################################
        ##### here's the code for all the following iterations
        ########################################################################################################################
        elif (command.startswith('pause') and state == State.RUNNING):
            print("INFO: PAUSE command received at " +
                  strftime("%Y-%m-%d %H:%M:%S", gmtime()) + " UTC")
            state = State.PAUSED
            #$# com.set_status(state)

            #$# # write new minor iteration number to file for the analyzers to read
            #$# f = open(alignConf.getProp("ItNrFile"), "w")
            #$# f.write(str(m_it))
            #$# f.close()

            # copy the saveset made by the analyzers in the last iteration to the desired place in workdir
            if magnFactorsMode == 0:
                #$#     histCopier.getHistoFile(n_it, m_it - tiltNamesLength)
                print(
                    "If not using histCopier, you should have put the root file manually into place by now."
                )
            else:
                #$#     histCopier.getHistoFile(n_it, m_it - 1)
                print(
                    "If not using histCopier, you should have put the root file manually into place by now."
                )
            sys.stdout.flush()

            allDone = False

            # if the minor iterations for one major iteration have finished perform fits etc:
            if (m_it % tiltNamesLength == 0):
                print("INFO:    minor iterations for major iteration " +
                      str(n_it) + " finished")
                print("INFO:    now merge the rootfiles and perform alignment")

                ### run the mirror Alignment - In Run 3 we have to feed these YAML via the .conf files prepared by RichAlignmentHelper
                start_time_RichMirrCombinFit = time()

                # copy pre-made reconstruction files needed to run Iterator_flat
                '''
                input_Iterator_flat = "/group/rich/pnaik/input_Iterator_flat/"
                for x in [
                        "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_md5000k_mapphiuse_conds_i0.yml",
                        "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_md5000k_mapphiuse_histos_i0.root",
                        "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_md5000k_mapphiuse_magnifs_priYn_predefined.txt",
                        "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_md5000k_mapphiuse_magnifs_priYp_predefined.txt",
                        "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_md5000k_mapphiuse_magnifs_priZn_predefined.txt",
                        "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_md5000k_mapphiuse_magnifs_priZp_predefined.txt",
                        "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_md5000k_mapphiuse_magnifs_secYn_predefined.txt",
                        "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_md5000k_mapphiuse_magnifs_secYp_predefined.txt",
                        "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_md5000k_mapphiuse_magnifs_secZn_predefined.txt",
                        "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_md5000k_mapphiuse_magnifs_secZp_predefined.txt",
                        #"hlt2_reco_rich2_panoptes.py" - we must not depend on this                                                ,
                        "rich2_fit_align_confs.yml",
                        #"rich2_iter_number.yml" - this will be 0 anyway                                                     ,
                        "rich2_reco_opts.yml",
                        "rich2_session_timestamp.yml",
                        "rich2_variant.yml"
                ]:
                    shutil.copyfile(input_Iterator_flat + x, workdir + x)
                '''
                '''
                if whichRich == 2:
                    input_Iterator_flat = "/group/rich/pnaik/input_Iterator_flat/Claire/"
                    for x in [
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_conds_i0.yml",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_conds_i1.yml",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_conds_i2.yml",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_conds_i3.yml",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_histos_i0.root",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_histos_i1.root",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_histos_i2.root",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_histos_i3.root",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_magnifs_priYn_predefined.txt",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_magnifs_priYp_predefined.txt",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_magnifs_priZn_predefined.txt",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_magnifs_priZp_predefined.txt",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_magnifs_secYn_predefined.txt",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_magnifs_secYp_predefined.txt",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_magnifs_secZn_predefined.txt",
                            "2022-07-20T21-10_rich2_minp60_qnt20f60_rndmpresc_unknown0k_magnifs_secZp_predefined.txt",
                            "rich2_fit_align_confs.yml",
                            #"rich2_iter_number.yml"                                                ,
                            "rich2_reco_opts.yml",
                            "rich2_session_timestamp.yml",
                            "rich2_variant.yml"
                    ]:
                        shutil.copyfile(input_Iterator_flat + x, workdir + x)
                    #
                '''
                if whichRich == 1:
                    input_Iterator_flat = "/group/rich/pnaik/input_Iterator_flat/Alex/"
                    for x in [
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_i0.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_i1.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_i2.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_i3.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_i4.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_i5.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_i6.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_i7.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_priYn_i0.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_priYp_i0.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_priZn_i0.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_priZp_i0.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_secYn_i0.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_secYp_i0.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_secZn_i0.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_conds_secZp_i0.yml",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_i0.root",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_i1.root",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_i2.root",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_i3.root",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_i4.root",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_i5.root",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_i6.root",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_i7.root",
                            #"2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_priYn_i0.root",
                            #"2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_priYp_i0.root",
                            #"2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_priZn_i0.root",
                            #"2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_priZp_i0.root",
                            #"2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_secYn_i0.root",
                            #"2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_secYp_i0.root",
                            #"2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_secZn_i0.root",
                            #"2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_histos_secZp_i0.root",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_magnifs_priYn_predefined.txt",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_magnifs_priYp_predefined.txt",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_magnifs_priZn_predefined.txt",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_magnifs_priZp_predefined.txt",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_magnifs_secYn_predefined.txt",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_magnifs_secYp_predefined.txt",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_magnifs_secZn_predefined.txt",
                            "2022-12-06T17-55_rich1_minp30_subset_Collision22_25k_Produce_magnifs_secZp_predefined.txt",
                            "rich1_fit_align_confs.yml",
                            #"rich2_iter_number.yml"                                                ,
                            "rich1_reco_opts.yml",
                            "rich1_session_timestamp.yml",
                            "rich1_variant.yml"
                    ]:
                        shutil.copyfile(input_Iterator_flat + x, workdir + x)
                    #
                if whichRich == 2:
                    input_Iterator_flat = "/group/rich/pnaik/input_Iterator_flat/Paras/"
                    for x in [
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_conds_i0.yml",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_histos_i0.root",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_conds_i1.yml",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_histos_i1.root",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_conds_i2.yml",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_histos_i2.root",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_conds_i3.yml",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_histos_i3.root",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_conds_i4.yml",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_histos_i4.root",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_conds_i5.yml",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_histos_i5.root",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_magnifs_priYn_predefined.txt",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_magnifs_priYp_predefined.txt",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_magnifs_priZn_predefined.txt",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_magnifs_priZp_predefined.txt",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_magnifs_secYn_predefined.txt",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_magnifs_secYp_predefined.txt",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_magnifs_secZn_predefined.txt",
                            "2022-12-06T17-55_rich2_minp60_subset_Collision22_200k_Produce_magnifs_secZp_predefined.txt",
                            "rich2_fit_align_confs.yml",
                            #"rich2_iter_number.yml"                                                ,
                            "rich2_reco_opts.yml",
                            "rich2_session_timestamp.yml",
                            "rich2_variant.yml"
                    ]:
                        shutil.copyfile(input_Iterator_flat + x, workdir + x)
                    #

                alignHelper.runMirrCombinFit(n_it)
                hh, r = divmod(time() - start_time_RichMirrCombinFit, 3600)
                mm, ss = divmod(r, 60)
                elapsed_runtimes['elapsed_times_RichMirrCombinFit'].append(
                    "{:0>2}:{:0>2}:{:05.2f}".format(
                        int(hh), int(mm), ss))  # RichMirrCombinFit run time

                enoughBinsPopulated = alignHelper.checkEnoughBinsPopulated(
                    n_it)
                print("INFO: enoughBinsPopulated = " +
                      str(enoughBinsPopulated))

                start_time_RichMirrAlign = time()
                alignHelper.runRichAlign(n_it)
                hh, r = divmod(time() - start_time_RichMirrAlign, 3600)
                mm, ss = divmod(r, 60)
                elapsed_runtimes['elapsed_times_RichMirrAlign'].append(
                    "{:0>2}:{:0>2}:{:05.2f}".format(
                        int(hh), int(mm), ss))  # RichMirrAlign run time

                ### pick up the new yml file from RichMirrAlign and format in in a way readable for the analyzers
                yaml = YAML()
                with open(workdir + '/' + 'rich' + str(whichRich) +
                          '_variant.yml') as finp:
                    variant = yaml.load(finp)
                newDBfilePre = workdir + variant + "_conds_i" + str(n_it + 1)
                if useDD4hep:
                    ymlHelper.reformatYAML(newDBfilePre + ".yml", workdir)
                else:
                    conversionsHelper.conv_YAML(newDBfilePre + ".yml")
                    ymlHelper.reformatYAML(newDBfilePre + ".yml", workdir)
                    xmlHelper.reformatXML(newDBfilePre + ".xml", workdir)

                ### check if the alignment has converged or failed
                conv = alignHelper.hasConverged()
                print("INFO: The alignment after Iteration", n_it,
                      "returns converged =", conv)

                if (conv or n_it >= maxIters):
                    allDone = True
                    if conv and n_it >= maxIters:
                        print(
                            "INFO: *** The alignment has reached its maximum number of iterations and has converged after Iteration "
                            + str(n_it) + " ***")
                    elif conv:
                        print(
                            "INFO: *** The alignment has converged after Iteration "
                            + str(n_it) + " ***")
                    else:
                        print(
                            "INFO: *** The alignment has reached its maximum number of iterations and has not converged ***"
                        )

                    elapsed_runtimes['elapsed_time_run'] = time(
                    ) - start_time_run  # Actual total iterating time
                    print(
                        "INFO: Elapsed time to run the Mirror Alignment set as",
                        elapsed_runtimes['elapsed_time_run'], "seconds.")
                    sys.stdout.flush()

                    # Run the monitoring, send plots to the Presenter, and generate PDF files;
                    #   this returns True if the monitoring sanity check failed.
                    if YAMLio:
                        ymlHelper.getCompareYAML(
                            compareDB)  ### can be different from starting-yml
                    else:
                        xmlHelper.getCompareXML(
                            compareDB)  ### can be different from starting-xml
                    insane = setupHelper.monitor(
                        n_it, compareDB, forcedUpdate_ChMagPol, flat=True)
                    #insane = False
                    print("INFO: alignMonitor.isInsane = " + str(insane))
                    sys.stdout.flush()

                    #nEventsListList = setupHelper.getNEventsListList(n_it) # won't work on MC
                    nEventsListList = [[10000000], 10000000]
                    print(
                        "INFO: average number of events processed per iteration = "
                        + str(nEventsListList[1]))

                    # Take action based on sanity checks
                    alert = 'INFO: Rich' + str(
                        whichRich
                    ) + ' mirror alignment is OK.'  # Default: The mirror alignment was in autoUpdate mode to start, ran completely normally, and does not request an update, OR autoUpdate mode was False to start.
                    sanitySubject = ""
                    sanityText = ""
                    if not enoughBinsPopulated:
                        if autoUpdate_initial:
                            autoUpdate = False
                            print(
                                "WARNING: There is poor photon population in at least one of the RICH mirror combinations. Mirror alignment "
                                + setupHelper.directoryTime +
                                " autoUpdate changed from True to False.")
                            if forcedUpdate_ChMagPol:
                                autoUpdateNoV = True
                                print(
                                    "WARNING: Since the polarity changed as well, we force autoUpdateNoV to be True (HLT2 gets no version)."
                                )
                                sanitySubject += " Polarity changed so no version provided to HLT2."
                                sanityText += "\n Since the polarity changed as well, no version was provided to HLT2. The mirror aligners should investigate."
                            setupHelper.sendEmail(
                                'autoUpdate disabled: Poor photon population in at least one of the RICH'
                                + str(whichRich) +
                                ' mirror combinations for RICH' +
                                str(whichRich) + ' alignment ' +
                                setupHelper.directoryTime + '.' +
                                sanitySubject,
                                'RICH' + str(whichRich) + ' mirror alignment '
                                + setupHelper.directoryTime +
                                ' autoUpdate changed from True to False.' +
                                sanityText, setupHelper.emails
                                if not testing else setupHelper.experts)
                            # This is not important enough to raise a WARNING to the Data Manager
                            alert = 'INFO: Rich' + str(
                                whichRich
                            ) + ' mirror alignment could not make a decision due to poor photon population in at least one of the RICH mirror combinations.'
                        else:
                            print(
                                "WARNING: There is poor photon population in at least one of the RICH mirror combinations; autoUpdate was already False."
                            )
                            setupHelper.sendEmail(
                                'Poor photon population in at least one of the RICH'
                                + str(whichRich) +
                                ' mirror combinations, autoUpdate was already False.',
                                'RICH' + str(whichRich) + ' mirror alignment '
                                + setupHelper.directoryTime +
                                ' autoUpdate already False.',
                                setupHelper.emails
                                if not testing else setupHelper.experts)
                    if nEventsListList[1] < int(
                            alignConf.getProp('requiredEvents')):
                        if autoUpdate_initial:
                            autoUpdate = False
                            print(
                                "WARNING: We processed an average number of events per iteration of "
                                + str(nEventsListList[1]) +
                                ", which is too low. Mirror alignment " +
                                setupHelper.directoryTime +
                                " autoUpdate changed from True to False. Investigate HLT farm availability for the LHCbA partition."
                            )
                            if forcedUpdate_ChMagPol:
                                autoUpdateNoV = True
                                print(
                                    "WARNING: Since the polarity changed as well, we force autoUpdateNoV to be True (HLT2 gets no version)."
                                )
                                sanitySubject += " Polarity changed so no version provided to HLT2."
                                sanityText += "\n Since the polarity changed as well, no version was provided to HLT2. The mirror aligners should investigate."
                            setupHelper.sendEmail(
                                'autoUpdate disabled: Lack of events processed for RICH'
                                + str(whichRich) + ' alignment ' +
                                setupHelper.directoryTime + '.' +
                                sanitySubject,
                                'RICH' + str(whichRich) + ' mirror alignment '
                                + setupHelper.directoryTime +
                                ' autoUpdate changed from True to False. \n Investigate HLT farm availability for the LHCbA partition. \n Average number of events processed per iteration: '
                                + str(nEventsListList[1]) + sanityText,
                                setupHelper.emails
                                if not testing else setupHelper.experts)
                            # This is not important enough to raise a WARNING to the Data Manager
                            alert = 'INFO: Rich' + str(
                                whichRich
                            ) + ' mirror alignment could not make a decision due to lack of processed events. Investigate HLT farm availability for the LHCbA partition.'  # There may have also been poor photon population.
                        else:
                            print(
                                "WARNING: We processed an average number of events per iteration of "
                                + str(nEventsListList[1]) +
                                ", which is too low; autoUpdate was already False."
                            )
                            setupHelper.sendEmail(
                                'Lack of events processed for RICH' +
                                str(whichRich) + ' alignment ' +
                                setupHelper.directoryTime +
                                ', autoUpdate was already False.',
                                'RICH' + str(whichRich) + ' mirror alignment '
                                + setupHelper.directoryTime +
                                ' autoUpdate already False. \n Average number of events processed per iteration: '
                                + str(nEventsListList[1]), setupHelper.emails
                                if not testing else setupHelper.experts)
                    if insane:
                        if autoUpdate_initial:
                            autoUpdate = False
                            print(
                                "WARNING: Tolerance sanity checks have failed; Mirror alignment "
                                + setupHelper.directoryTime +
                                " autoUpdate changed from True to False.")
                            if forcedUpdate_ChMagPol:
                                autoUpdateNoV = True
                                print(
                                    "WARNING: Since the polarity changed as well, we force autoUpdateNoV to be True (HLT2 gets no version)."
                                )
                                sanitySubject += " Polarity changed so no version provided to HLT2."
                                sanityText += " Since the polarity changed as well, no version was provided to HLT2. The mirror aligners should investigate."
                            setupHelper.sendEmail(
                                'autoUpdate disabled: Tolerance sanity checks failed for RICH'
                                + str(whichRich) + ' alignment ' +
                                setupHelper.directoryTime + '.' +
                                sanitySubject,
                                'RICH' + str(whichRich) + ' mirror alignment '
                                + setupHelper.directoryTime +
                                ' autoUpdate changed from True to False. \n Tolerance sanity checks have failed for this alignment.'
                                + sanityText, setupHelper.emails
                                if not testing else setupHelper.experts)
                            alert = 'ERROR: Rich' + str(
                                whichRich
                            ) + ' mirror alignment converged but constants changes are unreasonably large; 8-21h (21h-8h): please call (email) the Alignment piquet.'  # There may have also been a lack of processed events, or poor photon population.
                        else:
                            print(
                                "WARNING: Tolerance sanity checks have failed; autoUpdate was already False."
                            )
                            setupHelper.sendEmail(
                                'Tolerance sanity checks failed for RICH' +
                                str(whichRich) + ' alignment ' +
                                setupHelper.directoryTime +
                                ', autoUpdate was already False.',
                                'RICH' + str(whichRich) + ' mirror alignment '
                                + setupHelper.directoryTime +
                                ' autoUpdate already False. \n Tolerance sanity checks have failed for this alignment.',
                                setupHelper.emails
                                if not testing else setupHelper.experts)
                    sys.stdout.flush()

                    # if the alignment has converged AND there was no sanity problem AND we are not testing, overwrite the old polarity file with the new polarity file
                    if conv and not insane and not nEventsListList[1] < int(
                            alignConf.getProp(
                                'requiredEvents')) and not testing:
                        shutil.copyfile(
                            newPolarityFilename,  #
                            oldPolarityFilename)  #

                    if (forcedUpdate_ChMagPol
                            or updateHelper.updateDecision(n_it)) and conv:
                        # we now *always* use the newer YAML (XML) file [result of RichMirrAlign (result of RichMirrAlign, converted to XML)] when updating the mirror alignment, not the current YAML (XML)
                        if YAMLio:
                            shutil.copyfile(newDBfilePre + ".yml", lastDB)
                        else:
                            shutil.copyfile(newDBfilePre + ".xml", lastDB)
                        if autoUpdate:  # was initially True and is still True
                            if testing:
                                vN_DB_str = 'v' + str(
                                    int(latest_vN) + 1
                                ) + '_testing_' + setupHelper.directoryTime
                            else:
                                vN_DB_str = 'v' + str(int(latest_vN) + 1)
                            if YAMLio:
                                ymlHelper.finalYAML(vN_DB_str,
                                                    True)  # current + 1
                            else:
                                xmlHelper.finalXML(vN_DB_str,
                                                   True)  # current + 1
                            if testing:
                                setupHelper.finalize(  #
                                    vN_DB_str,  #
                                    conv,  #
                                    n_it,  #
                                    elapsed_time_config,  #
                                    elapsed_runtimes,  #
                                    None,  #
                                    nEventsListList,  #
                                    messageID,
                                    flat=True)  #
                            else:  # If not testing, and autoUpdate is still on, it's a good alignment.
                                setupHelper.finalize(  #
                                    vN_DB_str,  #
                                    conv,  #
                                    n_it,  #
                                    elapsed_time_config,  #
                                    elapsed_runtimes,  #
                                    True,  #
                                    nEventsListList,  #
                                    messageID,  #
                                    flat=True)  #
                                alert = 'INFO: Rich' + str(
                                    whichRich) + ' mirror alignment updated.'
                                updateHelper.updateDB(
                                    int(latest_vN) + 1, True,
                                    alert)  # current + 1
                        else:  # autoUpdate was never True, or was True but now switched to False
                            if testing:
                                vN_DB_str = 'v' + str(
                                    int(latest_vN) + 1
                                ) + '_testing_maybe_' + setupHelper.directoryTime
                            else:
                                vN_DB_str = 'v' + str(
                                    int(latest_vN) +
                                    1) + '_maybe_' + setupHelper.directoryTime
                            if YAMLio:
                                ymlHelper.finalYAML(
                                    vN_DB_str, True)  # (current + 1)_maybe
                            else:
                                xmlHelper.finalXML(vN_DB_str,
                                                   True)  # (current + 1)_maybe
                            setupHelper.finalize(
                                vN_DB_str,  #
                                conv,  #
                                n_it,  #
                                elapsed_time_config,  #
                                elapsed_runtimes,  #
                                None,  #
                                nEventsListList,  #
                                messageID,  #
                                flat=True)  #
                            if not testing:
                                if not autoUpdateNoV:  # provide the existing version ONLY if autoUpdate was always off, or autoUpdate was on but is now off *and* autoUpdateNoV remains False
                                    updateHelper.updateDB(
                                        int(latest_vN), False, alert)
                        #updateHelper.stopUpdateDIM()
                    else:
                        vN_DB_str = 'v' + str(int(latest_vN))
                        setupHelper.finalize(  #
                            vN_DB_str,  #
                            conv,  #
                            n_it,  #
                            elapsed_time_config,  #
                            elapsed_runtimes,  #
                            False,  #
                            nEventsListList,  #
                            messageID,  #
                            flat=True)  #
                        if not testing:
                            if autoUpdate_initial and not conv:
                                alert = 'ERROR: Rich' + str(
                                    whichRich
                                ) + ' mirror alignment failed to converge; 8-21h (21h-8h): please call (email) the Alignment piquet.'  # There may have also been a lack of processed events / sanity check failure.
                            updateHelper.updateDB(
                                int(latest_vN), False, alert)  # current
                            #updateHelper.stopUpdateDIM()
                    state = State.READY

                ### if the alignment has not converged and the maximal number of iterations has not been reached,
                ### set up everything for the next major iteration
                if n_it < maxIters and not conv:
                    n_it = n_it + 1
                    print("INFO:    ___________________________ ")
                    print("INFO:    Start new major Iteration " + str(n_it))
                    print("INFO:    *************************** ")

                    yaml = YAML()
                    with open(
                            workdir + 'rich' + str(whichRich) +
                            '_iter_number.yml', 'w') as fout:
                        yaml.dump(n_it, fout)

                    yaml = YAML()
                    with open(workdir + '/' + 'rich' + str(whichRich) +
                              '_variant.yml') as finp:
                        current_variant = yaml.load(finp)

                    ### get the yml-file produced by the last fit and create the ones with the tilted mirrors (xml for DetDesc, yml for DD4hep)
                    #currentMirrorDBFilePre = workdir + "Rich" + str(
                    #    whichRich
                    #) + "CondDBUpdate_" + thisNameStr + "_i" + str(n_it)
                    currentMirrorDBFilePre = workdir + current_variant + "_conds_i" + str(
                        n_it)
                    if not magnFactorsMode == 0:
                        if useDD4hep:
                            print(
                                "INFO: magnFactorsMode is nonzero. Creating coeff_calibration_yml files."
                            )
                            ymlHelper.create_coeff_calibration_yml_files(
                                currentMirrorDBFilePre + ".yml", n_it)
                        else:
                            print(
                                "INFO: magnFactorsMode is nonzero. Creating coeff_calibration_xml files."
                            )
                            xmlHelper.create_coeff_calibration_xml_files(
                                currentMirrorDBFilePre + ".xml", n_it)
                        sys.stdout.flush()

            ### just start the next minor iteration if not converged yet
            if n_it <= maxIters and not conv and not allDone:
                print("INFO: Major iteration " + str(n_it))
                print("INFO:    Minor iteration " + str(m_it))
                yaml = YAML()
                with open(workdir + '/' + 'rich' + str(whichRich) +
                          '_variant.yml') as finp:
                    variant = yaml.load(finp)
                if (m_it % tiltNamesLength == 0):
                    #pickupDB = workdir + "Rich" + str(
                    #    whichRich
                    #) + "CondDBUpdate_" + thisNameStr + "_i" + str(n_it)
                    pickupDB = workdir + variant + "_conds_i" + str(n_it)
                else:
                    #pickupDB = workdir + "Rich" + str(
                    #    whichRich
                    #) + "CondDBUpdate_" + thisNameStr + "_" + tiltNames[
                    #    m_it % tiltNamesLength] + "_i" + str(n_it)
                    pickupDB = workdir + variant + "_conds_" + tiltNames[
                        m_it % tiltNamesLength] + "_i" + str(n_it)
                if useDD4hep:
                    pickupDB = pickupDB + ".yml"
                else:
                    pickupDB = pickupDB + ".xml"
                shutil.copyfile(pickupDB, currentDB)
                if magnFactorsMode == 0:
                    m_it = m_it + tiltNamesLength
                else:
                    m_it = m_it + 1
                state = State.RUNNING
                sys.stdout.flush()
            sleep(2)

        elif command.startswith('stop') and state in (State.RUNNING,
                                                      State.READY):
            print("INFO: STOP command received at " +
                  strftime("%Y-%m-%d %H:%M:%S", gmtime()) + " UTC")
            elapsed_runtimes['elapsed_time_run'] = time(
            ) - start_time_run  # Actual iterating time
            print("INFO: Elapsed time to run the Mirror Alignment set as",
                  elapsed_runtimes['elapsed_time_run'], "seconds.")
            #$# nEventsListList = setupHelper.getNEventsListList(n_it)
            nEventsListList = [[10000000], 10000000]
            vN_DB_str = 'v' + str(int(latest_vN))
            # PN: Note to self, or ask Claire [depending on when this was put in]:
            #     Should we really be finalizing the alignment if a STOP occurs?
            setupHelper.finalize(
                vN_DB_str,  #
                conv,  #
                n_it,  #
                elapsed_time_config,  #
                elapsed_runtimes,  #
                False,  #
                nEventsListList,  #
                messageID,  #
                flat=True)  #
            if not testing:
                alert = 'INFO: Rich' + str(
                    whichRich
                ) + ' mirror alignment received an unexpected STOP command.'  # There may have also been a lack of processed events / sanity check failure.
                if autoUpdate_initial:
                    alert = 'ERROR: Rich' + str(
                        whichRich
                    ) + ' mirror alignment received an unexpected STOP command; 8-21h (21h-8h): please call (email) the Alignment piquet.'  # There may have also been a lack of processed events / sanity check failure.
                updateHelper.updateDB(int(latest_vN), False, alert)  # current
                #updateHelper.stopUpdateDIM()
            setupHelper.sendEmail(
                'STOP command received for RICH' + str(whichRich) +
                ' alignment ' + setupHelper.directoryTime + '.',
                'RICH' + str(whichRich) + ' mirror alignment ' +
                setupHelper.directoryTime +
                ' received an unexpected STOP command. \n Check for source of error, it may have involved a Traceback.',
                setupHelper.emails if not testing else setupHelper.experts)
            state = State.READY
            sys.stdout.flush()

        elif command.startswith('reset'):
            print("INFO: RESET command received at " +
                  strftime("%Y-%m-%d %H:%M:%S", gmtime()) + " UTC")
            sys.stdout.flush()
            state = State.NOT_READY
            break

        else:
            print("ERROR: Iterator_flat: bad transition from %s to %s" %
                  (state, command))
            sys.stdout.flush()
            state = State.ERROR
            break
        # Set our status
        #$# com.set_status(state)
        print("INFO: Iterator_flat state = " + str(state))


########################################################################################################################
##### This is where the party ends! (well, almost...)
########################################################################################################################

    print("INFO: End of run(whichRich = " + str(whichRich) +
          "). Storing alignment configurations and logs.")
    sys.stdout.flush()

    if not cancel:
        try:
            shutil.copyfile(saveConfiguration,
                            setupHelper.savedir + 'Configuration.py')
            print("INFO: Configuration.py stored in file: " +
                  setupHelper.savedir + 'Configuration.py')
        except:
            shutil.copyfile(
                saveConfiguration, loggingDir + 'Rich' + str(whichRich) +
                '_Configuration_' + setupHelper.directoryTime + '.py')
            print("INFO: Configuration.py stored in file: " + loggingDir +
                  'Rich' + str(whichRich) + '_Configuration_' +
                  setupHelper.directoryTime + '.py')
        try:
            shutil.copyfile(saveOnlineEnvBase,
                            setupHelper.savedir + 'OnlineEnvBase.py')
            print("INFO: OnlineEnvBase.py stored in file: " +
                  setupHelper.savedir + 'OnlineEnvBase.py')
        except:
            shutil.copyfile(
                saveOnlineEnvBase, loggingDir + 'Rich' + str(whichRich) +
                '_OnlineEnvBase_' + setupHelper.directoryTime + '.py')
            print("INFO: OnlineEnvBase.py stored in file: " + loggingDir +
                  'Rich' + str(whichRich) + '_OnlineEnvBase_' +
                  setupHelper.directoryTime + '.py')
    sys.stdout.flush()

    if not os.path.exists(setupHelper.savedir):
        print("INFO: Expect log file to be written to: " + loggingDir +
              'Rich' + str(whichRich) + '_' + setupHelper.directoryTime +
              '_hlt02.log')
        print("INFO: Expect reduced log file to be written to: " + loggingDir +
              'Rich' + str(whichRich) + '_' + setupHelper.directoryTime +
              '_hlt02reduced.log')
        if cancel:
            print("INFO: We choose to not execute the mirror alignment. ")
            print(
                "INFO: The input data is of mixed polarity and/or some it is magnet OFF data and/or some RICH calibrations are missing."
            )
        else:
            setupHelper.sendEmail(
                'ERROR: RICH' + str(whichRich) + ' mirror alignment ' +
                setupHelper.directoryTime + ' did not complete normally.' +
                ChMagPolSubject, 'ERROR: RICH' + str(whichRich) +
                ' mirror alignment ' + setupHelper.directoryTime +
                ' did not complete normally. \n No output directory was written. \n Log may be found on plus.cern.ch (via lbgw) in /group/rich/AlignmentFiles/Logging/ with timestamp '
                + setupHelper.directoryTime + '.' + ChMagPolText + ' \n Runs: '
                + str(runs) + ' \n Fills: ' + str(fills) + ' \n Polarities: ' +
                str(polsStr) + postText + ' \n dataVariant: ' +
                alignConf.getProp('dataVariant'),
                setupHelper.emails if not testing else setupHelper.experts)
            print(
                "ERROR: RICH" + str(whichRich) + " mirror alignment " +
                setupHelper.directoryTime +
                " did not complete normally. No output directory was written.")
    else:
        print("INFO: Expect log file to be written to: " +
              setupHelper.savedir + 'hlt02.log')
        print("INFO: Expect reduced log file to be written to: " +
              setupHelper.savedir + 'hlt02reduced.log')
    sys.stdout.flush()

    if os.path.exists(setupHelper.savedir):
        print("INFO: storing hlt02 log in file: " + setupHelper.savedir +
              "hlt02.log")
        print("INFO: storing reduced file: " + setupHelper.savedir +
              "hlt02reduced.log")
        sys.stdout.flush()
        shutil.copyfile(currentHLT02log, setupHelper.savedir + 'hlt02.log')
        # remove what we currently believe are extraneous items
        # print('INFO: Executing: ' + 'grep -vE "' + removeExtraneous + '" ' + setupHelper.savedir + 'hlt02.log > ' + setupHelper.savedir + 'hlt02reduced.log')
        os.system('grep -vE "' + removeExtraneous + '" ' +
                  setupHelper.savedir + 'hlt02.log > ' + setupHelper.savedir +
                  'hlt02reduced.log')
        lastAlignSuccessFile = open(lastAlignSuccess, 'w')
        lastAlignSuccessFile.write(
            'YES\n')  # python will convert \n to os.linesep
        lastAlignSuccessFile.close()
    else:
        print("INFO: storing hlt02 log in file: " + loggingDir + "Rich" +
              str(whichRich) + "_" + setupHelper.directoryTime + "_hlt02.log")
        print("INFO: storing reduced file: " + loggingDir + "Rich" +
              str(whichRich) + "_" + setupHelper.directoryTime +
              "_hlt02reduced.log")
        sys.stdout.flush()
        shutil.copyfile(
            currentHLT02log, loggingDir + 'Rich' + str(whichRich) + '_' +
            setupHelper.directoryTime + '_hlt02.log')
        # remove what we currently believe are extraneous items
        # print('INFO: Executing: ' + 'grep -vE "' + removeExtraneous + '" ' + loggingDir + 'Rich' + str(whichRich) + '_' + setupHelper.directoryTime + '_hlt02.log > ' + loggingDir + 'Rich' + str(whichRich) + '_' + setupHelper.directoryTime + '_hlt02reduced.log')
        os.system('grep -vE "' + removeExtraneous + '" ' + loggingDir +
                  'Rich' + str(whichRich) + '_' + setupHelper.directoryTime +
                  '_hlt02.log > ' + loggingDir + 'Rich' + str(whichRich) +
                  '_' + setupHelper.directoryTime + '_hlt02reduced.log')
        if cancel:
            lastAlignSuccessFile = open(lastAlignSuccess, 'w')
            lastAlignSuccessFile.write(
                'YES\n')  # python will convert \n to os.linesep
            lastAlignSuccessFile.close()

    # For good measure, make a reduced log in the logging directory
    os.system('grep -vE "' + removeExtraneous + '" ' + currentHLT02log +
              ' > ' + currentHLT02logReduced)

    print("INFO: Iterator_flat state set to " + str(state) + ".")
    print("INFO: Goodbye from the Rich" + str(whichRich) +
          " Mirror Alignment!")
    print("INFO: Claire says miezmiezmiez")  # Most important
    sys.stdout.flush()

    logProcess.kill()
    print("INFO: hlt02 logProcess for the mirror alignment has been killed.")
    sys.stdout.flush()

    # Set our status one last time
    print("INFO: Iterator_flat state = " + str(state))
    sleep(5)  # wait long enough for existing tasks to finish
    #$# com.set_status(state)

if __name__ == '__main__':
    run()
