###############################################################################
# (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
__version__ = ""
__author__ = "Alex Marshall <alex.marshall@cern.ch>, Paras Naik <Paras.Naik@cern.ch>, Markus Frank <Markus.Frank@cern.ch>"
# Maintainer: Alex Marshall, Paras Naik

# We will only use YAML in our Run 3 online reco
# now that DD4Hep can be run on DetDesc-generated MC

# CURRENTLY HARDCODED!!! FIX LATER WITH INPUT FROM ITERATOR
useAlignConds = False  # False means we run the reco using the current git DB conditions. True means we will use our mirror conditions.

import os, sys
# to use the git CondDB (PN: is this still good for Run 3?)
os.environ['GITCONDDBPATH'] = '/cvmfs/lhcb.cern.ch/lib/lhcb/git-conddb.test'

# Paras added these
import re, importlib, __builtin__

# PN: Let's presume Gaudi stuff is the same for Run 3
import Configurables as Configs
import Gaudi.Configuration as Gaudi
from Gaudi.Configuration import appendPostConfigAction
import GaudiKernel
from GaudiKernel.SystemOfUnits import GeV, MeV
from GaudiKernel.ProcessJobOptions import PrintOff, InstallRootLoggingHandler, logging
from Configurables import CondDB, GaudiSequencer, EventPersistencySvc, HistogramPersistencySvc, EventLoopMgr, OutputStream, Gaudi__SerializeCnvSvc, DstConf

from Communicator import *  # gets the State and Communicator classes

import time
import datetime
#PrintOff(999)
InstallRootLoggingHandler(level=logging.CRITICAL)

processingType = 'DataTaking'  # PN: This isn't used, candidate for removal

GaudiKernel.ProcessJobOptions._parser._parse_units(
    os.path.expandvars("$STDOPTS/units.opts"))
requirement = None

debug = 0


def dummy(*args, **kwd):  #PN: presume no change for Run 3
    pass


MSG_VERBOSE = 1
MSG_DEBUG = 2
MSG_INFO = 3
MSG_WARNING = 4
MSG_ERROR = 5
MSG_FATAL = 6
MSG_ALWAYS = 7


class saveHistograms(object):  #PN: presume no change for Run 3
    def __init__(self, runType, saveDir, n):
        self.__runType = runType
        self.__saveDir = saveDir
        self.__n = n

    def __call__(self):
        from Configurables import GaudiSequencer  #, UpdateAndReset
        app = Gaudi.ApplicationMgr()
        app.ExtSvc += ['MonitorSvc']

        from Configurables import MonitorSvc
        MonitorSvc().OutputLevel = MSG_ERROR
        MonitorSvc().UniqueServiceNames = 1
        MonitorSvc().CounterUpdateInterval = -1
        MonitorSvc().DimUpdateInterval = -1
        Online = importOnline()
        MonitorSvc().PartitionName = Online.PartitionName
        MonitorSvc().ProgramName = "AligWork_%s_%02d" % (self.__runType,
                                                         self.__n)


def staticVar(varname, value):  #PN: presume no change for Run 3
    def decorate(func):
        setattr(func, varname, value)
        return func

    return decorate


@staticVar("Online", None)
def importOnline():  #PN: presume no change for Run 3
    if importOnline.Online:
        print("INFO: importOnline.Online = ", importOnline.Online)
        return importOnline.Online
    if 'RUNINFO' in os.environ:
        runinfo = os.environ['RUNINFO']
        d, f = os.path.split(runinfo)
        name, ext = os.path.splitext(f)
        sys.path.insert(1, d)
        Online = importlib.import_module('OnlineEnv')
        sys.path.remove(d)
    else:
        import OnlineEnv as Online
    importOnline.Online = Online
    return Online


#===============================================================================
class ChangeP(
        object
):  #PN: Presume we WILL NOT need this for Run 3, but don't delete yet
    def __init__(self, minP):
        self.__minP = minP

    def __call__(self):
        from Configurables import PatForward, PatForwardTool
        for fwd in ("PatForwardHLT1", "PatForwardHLT2"):
            PatForward(fwd).addTool(PatForwardTool)
            PatForward(fwd).PatForwardTool.MinMomentum = self.__minP


#===============================================================================
def patchMoore(true_online_version, alignConf,
               n):  #configure Moore options for reconstruction
    whichRich = alignConf.getProp('Rich')
    print("INFO: (RichAnalyzer.py) whichRich = ", whichRich)
    workdir = alignConf.getProp('WorkDir')
    print("INFO: (RichAnalyzer.py) workdir = ", workdir)
    whichData = alignConf.getProp('dataVariant')
    print("INFO: (RichAnalyzer.py) whichData = ", whichData)

    today = datetime.date.today()
    CurrentYear = str(today.year)

    import GaudiConf.DstConf

    Online = importOnline()

    # Moore Configuration
    # PN: Here is where things will change for Run3
    # in the past moore was configured via Moore.Configuration
    # now it is configured via "options"
    # I comment out lines below
    # originating from moore = Moore.Configuration.Moore()
    # but **please leave them**,
    # We need to make sure that things match up at the end
    # We also need to find the appropriate print statements so that we
    # log every option in a similar way to before
    # I place PN comments to explain what was happening
    # I delete sections which are totally obsolete and we know what the equivalent is

    #PN: Run 2 configure moore
    #import Moore.Configuration
    #moore = Moore.Configuration.Moore()
    #PN: The filter code needed to process data with decisions from our line
    #moore.Hlt1FilterCode = "HLT_PASS_RE('Hlt1CalibRICHMirror(RICH%s)?Decision')" % whichRich

    #PN: we used to get DDDB and CONDDB tag from ConditionsMap
    #PN: now we will need to get them from somewhere else
    #import ConditionsMap
    #moore.OnlineMode = True
    #moore.DDDBtag = ConditionsMap.DDDBTag
    #PN: PRINT THE DDDB TAG
    #print("INFO: (RichAnalyzer.py) moore.DDDBtag = ", moore.DDDBtag)
    #moore.CondDBtag = ConditionsMap.CondDBTag
    #if ('Collision16' in whichData) or ('Collision15' in whichData):
    #    moore.CondDBtag = "cond-20161011"
    #PN: PRINT THE CONDDB TAG
    #print("INFO: (RichAnalyzer.py) moore.CondDBtag = ", moore.CondDBtag)

    if 'Collision' in whichData:
        # set MooreData type
        #moore.DataType = "20" + whichData[-2:]
        #PN: PRINT THE MOORE DATATYPE
        #print("INFO: (RichAnalyzer.py) moore.DataType = ", moore.DataType)
        inCurrentYear = (whichData[-2:] == CurrentYear[-2:])

    #PN: useDBsnapshot --- probably don't need this, but lets see
    #moore.UseDBSnapshot = True
    #PN: outputType = None --- probably don't need this, but lets see
    #moore.OutputType = 'NONE'

    #PN: get partition name
    #moore.PartitionName = os.environ.get('PARTITION_NAME', 'TEST')
    #PN: PRINT THE PARTITION NAME
    #print("INFO: (RichAnalyzer.py) moore.PartitionName = ",
    #      moore.PartitionName)

    # Turn off sequences we do not need
    # PN: These are Gaudi things
    # ---- who knows if still needed, we should find out
    def turnOffL0Decode():
        from Configurables import GaudiSequencer
        GaudiSequencer("DecodeTriggerSeq").Members = []

    appendPostConfigAction(turnOffL0Decode)

    def turnOffLumiSeq():
        from Configurables import GaudiSequencer
        GaudiSequencer("LumiSeq").Members = []

    appendPostConfigAction(turnOffLumiSeq)

    def turnOffNotPhysicsSeq():
        from Configurables import GaudiSequencer
        GaudiSequencer("NotPhysicsSeq").Members = []

    appendPostConfigAction(turnOffNotPhysicsSeq)

    # Selections
    from Configurables import MagneticFieldSvc
    MagneticFieldSvc().UseSetCurrent = True

    # PN: no idea what this is
    #moore.WriteFSR = False

    EventLoopMgr().OutputLevel = MSG_WARNING
    EventLoopMgr().Warnings = False

    # Need a postConfigAction to apply the correct condtions
    #PN: I hope we don't need this anymore to set the track momentum
    #minP = 20  # default: RICH1
    #if whichRich == 2:
    #    minP = 40
    #appendPostConfigAction(ChangeP(minP * GeV))

    from Configurables import TrackSys
    TrackSys().setProp("TrackPatRecAlgorithms",
                       ["FastVelo", "VeloTT", "ForwardHLT1", "ForwardHLT2"])

    #PN I presume we already have something similar for Run3
    #PN the key variable is whichRich, that comes from the Iterator/conf
    #if whichRich == 2:
    #    rConf.Radiators = ["Rich2Gas"]
    #if whichRich == 1:
    #    rConf.Radiators = ["Rich1Gas"]

    from Configurables import HistogramPersistencySvc
    HistogramPersistencySvc().OutputFile = ""

    #PN: We have to figure out how to do the equivalent to this in Run3
    #deltatheta = alignConf.getProp("deltaThetaWindow") / 1000.
    #RichAlign.DeltaThetaRange = [0.04, deltatheta, deltatheta]

    #PN: do these exist in the Moore options for Run 3?
    #moore.OutputLevel = MSG_INFO
    #moore.PrintFreq = 100

    #HistogramPersistencySvc().OutputLevel = MSG_ERROR
    from Configurables import CondDB
    from Configurables import CondDBAccessSvc
    from Configurables import ApplicationMgr
    from Configurables import HistogramPersistencySvc

    import uuid
    runType = "Rich" + str(whichRich) + "_" + str(uuid.uuid4()) + "_" + str(n)
    histoDir = workdir
    appendPostConfigAction(saveHistograms(runType, histoDir, n))

    # BEGIN CondDB Configuration YAML - PN this will all change

    # if useAlignConds:
    #     from Configurables import CondDB
    #     conddb = CondDB()
    #     conddb.Online = True
    #     conddb.EnableRunChangeHandler = True
    #
    #     conddb.UseDBSnapshot = False
    #     if ('Collision16' in whichData) or ('Collision15' in whichData):
    #         conddb.UseDBSnapshot = True
    #
    #     conddb.EnableRunStampCheck = False
    #
    #     # Here we must [in principle] exclude all years except for the current year, since previous years should be in some online database
    #     # PN: However, I have no idea how to access the online database for previous years... so for the years we were/are in the online DB, we're going to just keep using the "fake" Online DB
    #     #     ***so I am going to cheat here*** and continue to use the fake DB in all years that we were/are in the Online DB
    #     if not (('Collision16' in whichData) or ('Collision15' in whichData)):
    #         inCurrentYear = True
    #     if inCurrentYear:
    #         conddb.Tags[
    #             'ONLINE'] = Online.OnlDBTag  # needed when using gitDB (but only in the current year [in principle])
    #         print("INFO: (RichAnalyzer.py) conddb.Tags['ONLINE'] = ",
    #               conddb.Tags['ONLINE'])
    #
    #     # RunChangeHandler Configuration
    #
    #     # Dictionary of the main mirror alignment conditions
    #     mirrAlignConds = {
    #         'Rich1': [
    #             'Conditions/Alignment/Rich1/SecMirrorMasterTop_Align',  # not sure if this is really a main mirror alignment condition but for some reason is in this grouping
    #             'Conditions/Alignment/Rich1/SecMirrorMasterBot_Align'  # not sure if this is really a main mirror alignment condition but for some reason is in this grouping
    #         ] + [
    #             'Conditions/Alignment/Rich1/SphMirror%d_Align' % i
    #             for i in range(0, 4)
    #         ] + [
    #             'Conditions/Alignment/Rich1/SecMirror%d_Align' % i
    #             for i in range(0, 16)
    #         ],
    #         'Rich2': [] + [
    #             'Conditions/Alignment/Rich2/Rich2SphMirrorCont%d_Align' % i
    #             for i in range(0, 2)
    #         ] + [
    #             'Conditions/Alignment/Rich2/Rich2SecMirrorCont%d_Align' % i
    #             for i in range(0, 2)
    #         ] + [
    #             'Conditions/Alignment/Rich2/Rich2SphMirror%d_Align' % i
    #             for i in range(0, 56)
    #         ] + [
    #             'Conditions/Alignment/Rich2/Rich2SecMirror%d_Align' % i
    #             for i in range(0, 40)
    #         ]
    #     }
    #     #print("INFO: (RichAnalyzer.py) mirrAlignConds = ", mirrAlignConds)
    #
    #     # Dictionary of the other conditions that show up in our XML
    #     hpdConds = {
    #         'Rich1': ['Conditions/Alignment/Rich1/Rich1_Align'] + [
    #             'Conditions/Alignment/Rich1/HPD%d_Align' % i
    #             for i in range(0, 196)
    #         ] + [
    #             'Conditions/Alignment/Rich1/HPDPanel%d_Align' % i
    #             for i in range(0, 2)
    #         ],
    #         'Rich2': ['Conditions/Alignment/Rich2/Rich2_Align'] + [
    #             'Conditions/Alignment/Rich2/HPD%d_Align' % i
    #             for i in range(196, 484)
    #         ] + [
    #             'Conditions/Alignment/Rich2/HPDPanel%d_Align' % i
    #             for i in range(0, 2)
    #         ]
    #     }
    #     #print("INFO: (RichAnalyzer.py) hpdConds = ", hpdConds)
    #
    #     ## Create a Dictionary that combines mirrAlignConds and hpdConds
    #     allFileConds = {}
    #     for x, y in mirrAlignConds.items():
    #         if x in hpdConds.keys():
    #             allFileConds[x] = mirrAlignConds[x] + hpdConds[x]
    #         else:
    #             allFileConds[x] = mirrAlignConds[x]
    #     for x, y in hpdConds.items():
    #         if x not in mirrAlignConds.keys():
    #             allFileConds[x] = hpdConds[x]
    #     #print("INFO: (RichAnalyzer.py) allFileConds = ", allFileConds)
    #
    #     # Get the work directory usually /group/online/AligWork/Rich1/ or /group/online/AligWork/Rich2/
    #     if hasattr(Online, "AlignXmlDir"):
    #         xmlDir = Online.AlignXmlDir
    #     else:
    #         xmlDir = workdir
    #     print("INFO: (RichAnalyzer.py) xmlDir = ", xmlDir)
    #
    #     pat = os.path.join(xmlDir, "CondDB_%s.xml")
    #     print("INFO: (RichAnalyzer.py) pat = ", pat)
    #
    #     # baseloc = '/group/online/hlt/conditions'
    #     baseloc = CondDB().getProp("DBSnapshotDirectory")
    #     print("INFO: (RichAnalyzer.py) baseloc = ", baseloc)
    #
    #     ## Get the dictionary of conditions for the RunChangeHandler service
    #     ## from the file used by HLT2. Messing with sys.path is evil, but
    #     ## perhaps the least evil here...
    #     import sys
    #     try:
    #         import All
    #     except ImportError:
    #         rd = '/group/online/hlt/conditions/RunChangeHandler'
    #         sys.path.append(rd)
    #         import All
    #         print("INFO: (RichAnalyzer.py) Had to use sys.path.append(", rd,
    #               ") to make import All work.")
    #
    #     ### Add base path of the files to the files in All.py
    #     conditionmap = {
    #         os.path.join(baseloc, f): cs
    #         for f, cs in All.ConditionMap.iteritems()
    #     }
    #     #try:
    #     #  print("INFO: (RichAnalyzer.py) conditionmap = ", conditionmap)
    #     #except:
    #     #  print("INFO: (RichAnalyzer.py) conditionmap is too large to print. Try again on a single node.")
    #
    #     ## Create a specific conditions map with only mirrAlignConds for the Rich detector we are using
    #     mirrAlignConditionmap = {
    #         pat % sd: f
    #         for (sd, f) in mirrAlignConds.iteritems()
    #         if sd == ('Rich' + str(whichRich))
    #     }
    #
    #     #print("INFO: (RichAnalyzer.py) mirrAlignConditionmap = ", mirrAlignConditionmap)
    #
    #     # Configure RunChangeHandlerSvc by hand to allow different locations of the
    #     # files. postConfigAction hammer required...
    #     def applyConds():
    #         from Configurables import RunChangeHandlerSvc
    #         rch = RunChangeHandlerSvc()
    #         Gaudi.ApplicationMgr().ExtSvc.append(rch)
    #
    #         if inCurrentYear:
    #             rch.Conditions = {
    #                 c: f
    #                 for f, cs in conditionmap.iteritems() for c in cs
    #             }
    #             #try:
    #             #  print("INFO: (RichAnalyzer.py) rch.Conditions without all conditions in allFileConds removed and without our conditions added = ", rch.Conditions)
    #             #except:
    #             #  print("INFO: (RichAnalyzer.py) rch.Conditions without all conditions in allFileConds removed and without our conditions added is too large to print. Try again on a single node.")
    #         else:
    #             rch.Conditions = {
    #                 c: f
    #                 for f, cs in mirrAlignConditionmap.iteritems() for c in cs
    #             }
    #
    #         # pop all allFileConds conditions out of the rch.Conditions dictionary (we don't want Moore to access rich1mirroralign.xml or rich2mirroralign.xml in /group/online/hlt/conditions/LHCb/NoYear/first_three_digits_of_run/run/, where run is the run number)
    #         for c, f in (rch.Conditions.copy()).iteritems():
    #             for l in allFileConds.values():
    #                 if c in l:
    #                     rch.Conditions.pop(c, None)
    #         #try:
    #         #  print("INFO: (RichAnalyzer.py) rch.Conditions without our conditions added = ", rch.Conditions)
    #         #except:
    #         #  print("INFO: (RichAnalyzer.py) rch.Conditions without our conditions added is too large to print. Try again on a single node.")
    #
    #         # add *only* the main mirror alignment conditions for the Rich that we are aligning
    #         rch.Conditions.update({
    #             c: f
    #             for f, cs in mirrAlignConditionmap.iteritems() for c in cs
    #         })
    #         #try:
    #         #  print("INFO: (RichAnalyzer.py) rch.Conditions = ", rch.Conditions)
    #         #except:
    #         #  print("INFO: (RichAnalyzer.py) rch.Conditions is too large to print. Try again on a single node.")
    #
    #     #Send rch.Conditions to the RunChangeHandler
    #     appendPostConfigAction(applyConds)

    # END CondDB Configuration YAML

    # PRINT THE MOORE OR options OR equivalent
    #print("INFO: moore = ", moore)

    #PN: Return the object
    #return moore


def setupOnline(directory, filename, prefix, alignConf):
    import OnlineEnv as Online
    from Configurables import LHCb__FILEEvtSelector as es
    app = Gaudi.ApplicationMgr()
    app.AppName = ''
    app.HistogramPersistency = 'NONE'
    app.SvcOptMapping.append('LHCb::FILEEvtSelector/EventSelector')
    app.SvcOptMapping.append('LHCb::FmcMessageSvc/MessageSvc')
    from Configurables import MirrPauseHandler as mp
    mp().ReferenceRunNr = Online.DeferredRuns[0] if hasattr(
        Online, 'DeferredRuns') else 0
    mp().ItNrFile = alignConf.getProp('ItNrFile')
    app.TopAlg.append(
        "MirrPauseHandler")  #PN: Will we still use this in Run 3?
    sel = es("EventSelector")
    sel.AllowedRuns = Online.DeferredRuns if hasattr(Online,
                                                     'DeferredRuns') else []
    sel.Directory = directory
    sel.FilePrefix = prefix
    sel.Input = filename
    sel.DeleteFiles = False
    sel.Decode = False
    sel.Pause = True
    sel.PauseSleep = 5000
    app.EvtSel = sel
    app.EvtMax = alignConf.getProp('EvtMax')
    Online.rawPersistencySvc()
    evtloop = Configs.EventLoopMgr('EventLoopMgr')
    evtloop.Warnings = False
    evtloop.EvtSel = sel
    app.EventLoop = evtloop
    runable = Configs.LHCb__EventRunable("Runable")
    runable.MEPManager = ""
    app.Runable = runable
    app.AuditAlgorithms = False
    Configs.MonitorSvc().OutputLevel = MSG_ERROR
    Configs.MonitorSvc().UniqueServiceNames = 1
    Configs.MonitorSvc().DimUpdateInterval = -1
    Configs.RootHistCnv__PersSvc("RootHistSvc").OutputLevel = MSG_ERROR
    app.OutputLevel = MSG_INFO
    print("INFO: sel = ", sel)


#============================================================================================================
def patchMessages():
    import OnlineEnv as Online
    app = Gaudi.ApplicationMgr()
    Configs.AuditorSvc().Auditors = []
    app.MessageSvcType = 'LHCb::FmcMessageSvc'
    if Gaudi.allConfigurables.has_key('MessageSvc'):
        del Gaudi.allConfigurables['MessageSvc']
    msg = Configs.LHCb__FmcMessageSvc('MessageSvc')
    msg.fifoPath = os.environ['LOGFIFO']
    msg.LoggerOnly = True
    msg.doPrintAlways = False
    msg.OutputLevel = MSG_INFO  # Online.OutputLevel


#============================================================================================================
def start():
    Online = importOnline()
    import sys
    Online.end_config(False)


#============================================================================================================
def doIt(filename="", n=0, whichRich=1):  #PN: The main bit

    if whichRich == 1:
        from Configurables import Rich1MirrAlignOnConf
        alignConf = Rich1MirrAlignOnConf()
    elif whichRich == 2:
        from Configurables import Rich2MirrAlignOnConf
        alignConf = Rich2MirrAlignOnConf()
    alignConf.__apply_configuration__()

    workdir = alignConf.getProp('WorkDir')
    true_online = os.environ.has_key('LOGFIFO') and (
        os.environ.has_key('PARTITION')
        or os.environ.has_key('PARTITION_NAME'))

    if os.path.isdir(filename):
        directory = filename
        prefix = 'Run_'
    else:
        directory = os.path.dirname(filename)
        prefix = os.path.basename(filename)[:4]

    br = patchMoore(true_online, alignConf, n)
    setupOnline(directory, filename, prefix, alignConf)
    if true_online:
        patchMessages()
    start()
